/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.controls.playList;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;

import net.sourceforge.atunes.gui.Fonts;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.PlayListTableModel;
import net.sourceforge.atunes.gui.model.TransferableList;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.ClearPlayListAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedAlbumsAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedArtistsAction;
import net.sourceforge.atunes.kernel.actions.LoadPlayListAction;
import net.sourceforge.atunes.kernel.actions.MoveDownAction;
import net.sourceforge.atunes.kernel.actions.MoveToBottomAction;
import net.sourceforge.atunes.kernel.actions.MoveToTopAction;
import net.sourceforge.atunes.kernel.actions.MoveUpAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromPlayListAction;
import net.sourceforge.atunes.kernel.actions.SavePlayListAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteAlbumAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteArtistAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteSongAction;
import net.sourceforge.atunes.kernel.actions.ShowPlayListItemInfoAction;
import net.sourceforge.atunes.kernel.actions.ShufflePlayListAction;
import net.sourceforge.atunes.kernel.actions.TogglePlayListFilterAction;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class PlayListTable.
 */
public class PlayListTable extends JTable implements DragSourceListener, DragGestureListener {

	/**
	 * The Enum PlayState.
	 */
	public enum PlayState {

		/** The STOPPED. */
		STOPPED,

		/** The PLAYING. */
		PLAYING,

		/** The PAUSED. */
		PAUSED
	}

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 9209069236823917569L;

	/** The playing song. */
	private int playingSong;

	/** The play state. */
	private PlayState playState = PlayState.STOPPED;

	/** The menu. */
	private JPopupMenu menu;

	/** The right menu. */
	JPopupMenu rightMenu;

	/** The play item. */
	private JMenuItem playItem;

	/** The open folder. */
	private JMenuItem openFolder;

	/** The tag menu. */
	private JMenu tagMenu;

	/** The edit tag item. */
	private JMenuItem editTagItem;

	/** The auto set lyrics item. */
	private JMenuItem autoSetLyricsItem;

	/** The auto set track number item. */
	private JMenuItem autoSetTrackNumberItem;

	/** The auto set genre item. */
	private JMenuItem autoSetGenreItem;

	/** The auto set title item. */
	private JMenuItem autoSetTitleItem;

	/** The save item. */
	private JMenuItem saveItem;

	/** The load item. */
	private JMenuItem loadItem;

	/** The filter item. */
	private JMenuItem filterItem;

	/** The top item. */
	private JMenuItem topItem;

	/** The up item. */
	private JMenuItem upItem;

	/** The delete item. */
	private JMenuItem deleteItem;

	/** The down item. */
	private JMenuItem downItem;

	/** The bottom item. */
	private JMenuItem bottomItem;

	/** The shuffle item. */
	private JMenuItem shuffleItem;

	/** The info item. */
	private JMenuItem infoItem;

	/** The clear item. */
	private JMenuItem clearItem;

	/** The favorite song. */
	private JMenuItem favoriteSong;

	/** The favorite artist. */
	private JMenuItem favoriteArtist;

	/** The favorite album. */
	private JMenuItem favoriteAlbum;

	/** The artist item. */
	private JMenuItem artistItem;

	/** The album item. */
	private JMenuItem albumItem;

	/** The arrange columns. */
	private JMenuItem arrangeColumns;

	/** The smart play list. */
	private JMenuItem smartPlayList;

	/** The add10 random songs. */
	private JMenuItem add10RandomSongs;

	/** The add50 random songs. */
	private JMenuItem add50RandomSongs;

	/** The add100 random songs. */
	private JMenuItem add100RandomSongs;

	/** The add10 songs most played. */
	private JMenuItem add10SongsMostPlayed;

	/** The add50 songs most played. */
	private JMenuItem add50SongsMostPlayed;

	/** The add100 songs most played. */
	private JMenuItem add100SongsMostPlayed;

	/** The add1 artists most played. */
	private JMenuItem add1ArtistsMostPlayed;

	/** The add5 artists most played. */
	private JMenuItem add5ArtistsMostPlayed;

	/** The add10 artists most played. */
	private JMenuItem add10ArtistsMostPlayed;

	/** The add1 albums most played. */
	private JMenuItem add1AlbumsMostPlayed;

	/** The add5 albums most played. */
	private JMenuItem add5AlbumsMostPlayed;

	/** The add10 albums most played. */
	private JMenuItem add10AlbumsMostPlayed;

	/** The add10 unplayed songs. */
	private JMenuItem add10UnplayedSongs;

	/** The add50 unplayed songs. */
	private JMenuItem add50UnplayedSongs;

	/** The add100 unplayed songs. */
	private JMenuItem add100UnplayedSongs;

	/** The listeners. */
	List<PlayListColumnClickedListener> listeners = new ArrayList<PlayListColumnClickedListener>();

	/**
	 * Drag source for this play list to drag songs to device
	 */
	private DragSource dragSource;

	/**
	 * Instantiates a new play list table.
	 */
	public PlayListTable() {
		super();
		setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		setShowGrid(false);

		// Add mouse click listener on table header
		getTableHeader().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				// Use left button to sort
				if (e.getButton() == MouseEvent.BUTTON1) {
					PlayListColumnModel colModel = (PlayListColumnModel) getColumnModel();
					int column = colModel.getColumnIndexAtPosition(e.getX());
					if (column >= 0) {
						Column columnClicked = ((PlayListTableModel) getModel()).getColumn(column);
						for (int i = 0; i < listeners.size(); i++) {
							listeners.get(i).columnClicked(columnClicked);
						}
					}
				}
				// Use right button to arrange columns
				else if (e.getButton() == MouseEvent.BUTTON3) {
					rightMenu.show(PlayListTable.this, e.getX(), e.getY());
				}
			}
		});

		// Set table model
		setModel(new PlayListTableModel(this));

		// Set column model
		setColumnModel(new PlayListColumnModel(this));

		// Disable autoresize, as we will control it
		setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		// Set renderers
		PlayListRenderers.addRenderers(this);

		// Remove enter key event, which moves selection down
		getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "none");

		rightMenu = new JPopupMenu();
		arrangeColumns = setMenuItem(new JMenuItem(LanguageTool.getString("ARRANGE_COLUMNS")));
		rightMenu.add(arrangeColumns);

		menu = new JPopupMenu();

		playItem = setMenuItem(new JMenuItem(LanguageTool.getString("PLAY"), ImageLoader.PLAY_MENU));

		openFolder = setMenuItem(new JMenuItem(LanguageTool.getString("OPEN_FOLDER"), ImageLoader.FOLDER_ORANGE));

		tagMenu = setMenu(new JMenu(LanguageTool.getString("TAGS")));
		editTagItem = setMenuItem(new JMenuItem(LanguageTool.getString("EDIT_TAG")));
		autoSetTrackNumberItem = setMenuItem(new JMenuItem(LanguageTool.getString("AUTO_SET_TRACK_NUMBER")));
		autoSetLyricsItem = setMenuItem(new JMenuItem(LanguageTool.getString("AUTO_SET_LYRICS")));
		autoSetGenreItem = setMenuItem(new JMenuItem(LanguageTool.getString("AUTO_SET_GENRE")));
		autoSetTitleItem = setMenuItem(new JMenuItem(LanguageTool.getString("AUTO_SET_TITLE")));
		tagMenu.add(editTagItem);
		tagMenu.add(autoSetLyricsItem);
		tagMenu.add(autoSetTrackNumberItem);
		tagMenu.add(autoSetTitleItem);
		tagMenu.add(autoSetTrackNumberItem);
		tagMenu.add(autoSetGenreItem);

		saveItem = setMenuItem(new JMenuItem(Actions.getAction(SavePlayListAction.class)));
		loadItem = setMenuItem(new JMenuItem(Actions.getAction(LoadPlayListAction.class)));
		filterItem = setMenuItem(new JMenuItem(Actions.getAction(TogglePlayListFilterAction.class)));
		topItem = setMenuItem(new JMenuItem(Actions.getAction(MoveToTopAction.class)));
		upItem = setMenuItem(new JMenuItem(Actions.getAction(MoveUpAction.class)));
		deleteItem = setMenuItem(new JMenuItem(Actions.getAction(RemoveFromPlayListAction.class)));
		downItem = setMenuItem(new JMenuItem(Actions.getAction(MoveDownAction.class)));
		bottomItem = setMenuItem(new JMenuItem(Actions.getAction(MoveToBottomAction.class)));
		shuffleItem = setMenuItem(new JMenuItem(Actions.getAction(ShufflePlayListAction.class)));
		infoItem = setMenuItem(new JMenuItem(Actions.getAction(ShowPlayListItemInfoAction.class)));
		clearItem = setMenuItem(new JMenuItem(Actions.getAction(ClearPlayListAction.class)));
		favoriteSong = setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteSongAction.class)));
		favoriteAlbum = setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteAlbumAction.class)));
		favoriteArtist = setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteArtistAction.class)));
		artistItem = setMenuItem(new JMenuItem(Actions.getAction(CreatePlayListWithSelectedArtistsAction.class)));
		albumItem = setMenuItem(new JMenuItem(Actions.getAction(CreatePlayListWithSelectedAlbumsAction.class)));

		smartPlayList = new JMenu(LanguageTool.getString("SMART_PLAYLIST"));
		add10RandomSongs = new JMenuItem(LanguageTool.getString("ADD_10_RANDOM_SONGS"));
		add50RandomSongs = new JMenuItem(LanguageTool.getString("ADD_50_RANDOM_SONGS"));
		add100RandomSongs = new JMenuItem(LanguageTool.getString("ADD_100_RANDOM_SONGS"));
		smartPlayList.add(add10RandomSongs);
		smartPlayList.add(add50RandomSongs);
		smartPlayList.add(add100RandomSongs);
		smartPlayList.add(new JSeparator());
		add10SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_SONGS_MOST_PLAYED"));
		add50SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_50_SONGS_MOST_PLAYED"));
		add100SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_100_SONGS_MOST_PLAYED"));
		smartPlayList.add(add10SongsMostPlayed);
		smartPlayList.add(add50SongsMostPlayed);
		smartPlayList.add(add100SongsMostPlayed);
		smartPlayList.add(new JSeparator());
		add1AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_ALBUM_MOST_PLAYED"));
		add5AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_5_ALBUMS_MOST_PLAYED"));
		add10AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_ALBUMS_MOST_PLAYED"));
		smartPlayList.add(add1AlbumsMostPlayed);
		smartPlayList.add(add5AlbumsMostPlayed);
		smartPlayList.add(add10AlbumsMostPlayed);
		smartPlayList.add(new JSeparator());
		add1ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_ARTIST_MOST_PLAYED"));
		add5ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_5_ARTISTS_MOST_PLAYED"));
		add10ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_ARTISTS_MOST_PLAYED"));
		smartPlayList.add(add1ArtistsMostPlayed);
		smartPlayList.add(add5ArtistsMostPlayed);
		smartPlayList.add(add10ArtistsMostPlayed);
		add10UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_10_UNPLAYED_SONGS"));
		add50UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_50_UNPLAYED_SONGS"));
		add100UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_100_UNPLAYED_SONGS"));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(add10UnplayedSongs);
		smartPlayList.add(add50UnplayedSongs);
		smartPlayList.add(add100UnplayedSongs);

		menu.add(playItem);
		menu.add(infoItem);
		menu.add(new JSeparator());
		menu.add(openFolder);
		menu.add(new JSeparator());
		menu.add(tagMenu);
		menu.add(new JSeparator());
		menu.add(saveItem);
		menu.add(loadItem);
		menu.add(new JSeparator());
		menu.add(filterItem);
		menu.add(new JSeparator());
		menu.add(deleteItem);
		menu.add(clearItem);
		menu.add(new JSeparator());
		menu.add(topItem);
		menu.add(upItem);
		menu.add(downItem);
		menu.add(bottomItem);
		menu.add(new JSeparator());
		menu.add(shuffleItem);
		menu.add(new JSeparator());
		menu.add(smartPlayList);
		menu.add(new JSeparator());
		menu.add(favoriteSong);
		menu.add(favoriteAlbum);
		menu.add(favoriteArtist);
		menu.add(new JSeparator());
		menu.add(artistItem);
		menu.add(albumItem);

		GuiUtils.applyComponentOrientation(menu);

		// Create drag source and set listener
		dragSource = new DragSource();
		dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY, this);

	}

	/**
	 * Adds the play list column clicked listener.
	 * 
	 * @param l
	 *            the l
	 */
	public void addPlayListColumnClickedListener(PlayListColumnClickedListener l) {
		listeners.add(l);
	}

	/**
	 * Gets the add100 random songs.
	 * 
	 * @return the add100RandomSongs
	 */
	public JMenuItem getAdd100RandomSongs() {
		return add100RandomSongs;
	}

	/**
	 * Gets the add100 songs most played.
	 * 
	 * @return the add100SongsMostPlayed
	 */
	public JMenuItem getAdd100SongsMostPlayed() {
		return add100SongsMostPlayed;
	}

	/**
	 * Gets the add10 albums most played.
	 * 
	 * @return the add10AlbumsMostPlayed
	 */
	public JMenuItem getAdd10AlbumsMostPlayed() {
		return add10AlbumsMostPlayed;
	}

	/**
	 * Gets the add10 artists most played.
	 * 
	 * @return the add10ArtistsMostPlayed
	 */
	public JMenuItem getAdd10ArtistsMostPlayed() {
		return add10ArtistsMostPlayed;
	}

	/**
	 * Gets the add10 random songs.
	 * 
	 * @return the add10RandomSongs
	 */
	public JMenuItem getAdd10RandomSongs() {
		return add10RandomSongs;
	}

	/**
	 * Gets the add10 songs most played.
	 * 
	 * @return the add10SongsMostPlayed
	 */
	public JMenuItem getAdd10SongsMostPlayed() {
		return add10SongsMostPlayed;
	}

	/**
	 * Gets the add1 albums most played.
	 * 
	 * @return the add1AlbumsMostPlayed
	 */
	public JMenuItem getAdd1AlbumsMostPlayed() {
		return add1AlbumsMostPlayed;
	}

	/**
	 * Gets the add1 artists most played.
	 * 
	 * @return the add1ArtistsMostPlayed
	 */
	public JMenuItem getAdd1ArtistsMostPlayed() {
		return add1ArtistsMostPlayed;
	}

	/**
	 * Gets the add50 random songs.
	 * 
	 * @return the add50RandomSongs
	 */
	public JMenuItem getAdd50RandomSongs() {
		return add50RandomSongs;
	}

	/**
	 * Gets the add50 songs most played.
	 * 
	 * @return the add50SongsMostPlayed
	 */
	public JMenuItem getAdd50SongsMostPlayed() {
		return add50SongsMostPlayed;
	}

	/**
	 * Gets the add5 albums most played.
	 * 
	 * @return the add5AlbumsMostPlayed
	 */
	public JMenuItem getAdd5AlbumsMostPlayed() {
		return add5AlbumsMostPlayed;
	}

	/**
	 * Gets the add5 artists most played.
	 * 
	 * @return the add5ArtistsMostPlayed
	 */
	public JMenuItem getAdd5ArtistsMostPlayed() {
		return add5ArtistsMostPlayed;
	}

	/**
	 * Gets the arrange columns.
	 * 
	 * @return the arrangeColumns
	 */
	public JMenuItem getArrangeColumns() {
		return arrangeColumns;
	}

	/**
	 * Gets the auto set genre item.
	 * 
	 * @return the auto set genre item
	 */
	public JMenuItem getAutoSetGenreItem() {
		return autoSetGenreItem;
	}

	/**
	 * Gets the auto set lyrics item.
	 * 
	 * @return the auto set lyrics item
	 */
	public JMenuItem getAutoSetLyricsItem() {
		return autoSetLyricsItem;
	}

	/**
	 * Gets the auto set title item.
	 * 
	 * @return the auto set title item
	 */
	public JMenuItem getAutoSetTitleItem() {
		return autoSetTitleItem;
	}

	/**
	 * Gets the auto set track item.
	 * 
	 * @return the auto set track item
	 */
	public JMenuItem getAutoSetTrackItem() {
		return autoSetTrackNumberItem;
	}

	/**
	 * Gets the edits the tag item.
	 * 
	 * @return the edits the tag item
	 */
	public JMenuItem getEditTagItem() {
		return editTagItem;
	}

	/**
	 * Gets the menu.
	 * 
	 * @return the menu
	 */
	public JPopupMenu getMenu() {
		return menu;
	}

	/**
	 * Gets the playing song.
	 * 
	 * @return the playing song
	 */
	public int getPlayingSong() {
		return playingSong;
	}

	/**
	 * Gets the play item.
	 * 
	 * @return the play item
	 */
	public JMenuItem getPlayItem() {
		return playItem;
	}

	/**
	 * Gets the play state.
	 * 
	 * @return the play state
	 */
	public PlayState getPlayState() {
		return playState;
	}

	/**
	 * Sets the menu.
	 * 
	 * @param item
	 *            the item
	 * 
	 * @return the j menu
	 */
	private JMenu setMenu(JMenu item) {
		item.setFont(Fonts.PLAY_LIST_FONT);
		return item;
	}

	/**
	 * Sets the menu item.
	 * 
	 * @param item
	 *            the item
	 * 
	 * @return the j menu item
	 */
	private JMenuItem setMenuItem(JMenuItem item) {
		item.setFont(Fonts.PLAY_LIST_FONT);
		return item;
	}

	/**
	 * Sets the playing song.
	 * 
	 * @param row
	 *            the new playing song
	 */
	public void setPlayingSong(int row) {
		playingSong = row;
		((PlayListTableModel) getModel()).refreshTable();
	}

	/**
	 * Sets the play state.
	 * 
	 * @param playState
	 *            the new play state
	 */
	public void setPlayState(PlayState playState) {
		this.playState = playState;
		((PlayListTableModel) getModel()).refreshTable();
	}

	/**
	 * Gets the add10 unplayed songs.
	 * 
	 * @return the add10 unplayed songs
	 */
	public JMenuItem getAdd10UnplayedSongs() {
		return add10UnplayedSongs;
	}

	/**
	 * Gets the add50 unplayed songs.
	 * 
	 * @return the add50 unplayed songs
	 */
	public JMenuItem getAdd50UnplayedSongs() {
		return add50UnplayedSongs;
	}

	/**
	 * Gets the add100 unplayed songs.
	 * 
	 * @return the add100 unplayed songs
	 */
	public JMenuItem getAdd100UnplayedSongs() {
		return add100UnplayedSongs;
	}

	/**
	 * Gets the open folder.
	 * 
	 * @return the openFolder
	 */
	public JMenuItem getOpenFolder() {
		return openFolder;
	}

	@Override
	public void dragDropEnd(DragSourceDropEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragEnter(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragExit(DragSourceEvent dse) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragGestureRecognized(DragGestureEvent dge) {
		// Get selected rows, add AudioFile objects to a list and start a drag event
		int[] selectedRows = getSelectedRows();
		List<Object> itemsToDrag = new ArrayList<Object>();
		for (int i : selectedRows) {
			AudioObject obj = ((PlayListTableModel) getModel()).getFileAt(i);
			if (obj instanceof AudioFile) {
				itemsToDrag.add(obj);
			}
		}
		TransferableList<Object> items = new TransferableList<Object>(itemsToDrag);
		dragSource.startDrag(dge, DragSource.DefaultCopyDrop, items, this);
	}

	@Override
	public void dragOver(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dropActionChanged(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}

}
