/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.editTagDialog;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.swing.ImageIcon;
import javax.swing.SwingWorker;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.AutoCompleteFeature;
import net.sourceforge.atunes.gui.views.dialogs.EditTagDialog;
import net.sourceforge.atunes.kernel.controllers.model.DialogController;
import net.sourceforge.atunes.kernel.executors.BackgroundExecutor;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.DefaultTag;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.Tag;
import net.sourceforge.atunes.kernel.utils.AudioFilePictureUtils;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.kernel.utils.SystemProperties.OperatingSystem;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

import org.jdesktop.swingx.combobox.ListComboBoxModel;

/**
 * The Class EditTagDialogController.
 */
public class EditTagDialogController extends DialogController<EditTagDialog> {

	Logger logger = new Logger();

	/** The audio files editing. */
	List<AudioFile> audioFilesEditing;

	private boolean shouldEditCover;
	private byte[] newCover;

	/**
	 * Instantiates a new edits the tag dialog controller.
	 * 
	 * @param dialog
	 *            the dialog
	 */
	public EditTagDialogController(EditTagDialog dialog) {
		super(dialog);
		addBindings();
		addStateBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		// Add genres combo box items
		List<String> genresSorted = Arrays.asList(Tag.genres);
		Collections.sort(genresSorted);
		getDialogControlled().getGenreComboBox().setModel(new ListComboBoxModel(genresSorted));
		// Add autocompletion
		new AutoCompleteFeature(getDialogControlled().getGenreComboBox());

		EditTagDialogActionListener actionListener = new EditTagDialogActionListener(this, getDialogControlled());
		getDialogControlled().getOkButton().addActionListener(actionListener);
		getDialogControlled().getCancelButton().addActionListener(actionListener);

		getDialogControlled().getCoverButton().addActionListener(actionListener);
		getDialogControlled().getRemoveCoverButton().addActionListener(actionListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/**
	 * Edits the files.
	 * 
	 * @param files
	 *            the files
	 */

	public void editFiles(final List<AudioFile> files) {
		audioFilesEditing = files;

		getDialogControlled().getCover().setIcon(null);
		getDialogControlled().getCover().setEnabled(false);
		getDialogControlled().getCoverButton().setEnabled(false);
		getDialogControlled().getRemoveCoverButton().setEnabled(false);
		shouldEditCover = false;
		newCover = null;

		// Load artists into combo box
		List<Artist> artistList = RepositoryHandler.getInstance().getArtists();
		List<String> artistNames = new ArrayList<String>();
		for (Artist a : artistList) {
			artistNames.add(a.getName());
		}
		getDialogControlled().getArtistTextField().setModel(new ListComboBoxModel(artistNames));
		// Activate autocompletion of artists
		new AutoCompleteFeature(getDialogControlled().getArtistTextField());

		// Load albums into combo box
		List<Album> albumList = RepositoryHandler.getInstance().getAlbums();
		List<String> albumNames = new ArrayList<String>();
		for (Album alb : albumList) {
			// Because of artists and album artists there can be more than one album with the same name
			if (!albumNames.contains(alb.getName())) {
				albumNames.add(alb.getName());
			}
		}
		getDialogControlled().getAlbumTextField().setModel(new ListComboBoxModel(albumNames));
		// Active autocompletion of albums
		new AutoCompleteFeature(getDialogControlled().getAlbumTextField());

		if (files.size() == 1) {
			getDialogControlled().getTrackNumberTextField().setEnabled(true);
			Tag tag = files.get(0).getTag();
			if (tag != null) {
				getDialogControlled().getTitleTextField().setText(tag.getTitle());
				getDialogControlled().getTitleTextField().setEnabled(true);
				getDialogControlled().getTrackNumberTextField().setText(Integer.toString(((DefaultTag) tag).getTrackNumber()));
				getDialogControlled().getCover().setEnabled(false);
			} else {
				getDialogControlled().getTitleTextField().setText("");
				getDialogControlled().getTitleTextField().setEnabled(true);
				getDialogControlled().getTrackNumberTextField().setText("");
				getDialogControlled().getCover().setEnabled(false);
			}

		} else {
			getDialogControlled().getTitleTextField().setText("");
			getDialogControlled().getTitleTextField().setEnabled(false);
			getDialogControlled().getTrackNumberTextField().setText("");
			getDialogControlled().getTrackNumberTextField().setEnabled(false);
		}

		// Check if at least one audio file supports internal pictures
		boolean supportsInternalPicture = false;
		for (AudioFile af : audioFilesEditing) {
			if (af.supportsInternalPicture()) {
				supportsInternalPicture = true;
				break;
			}
		}
		if (supportsInternalPicture) {
			new SwingWorker<ImageIcon, Void>() {
				@Override
				protected ImageIcon doInBackground() throws Exception {
					return files.size() == 1 ? AudioFilePictureUtils.getInsidePicture(files.get(0), Constants.DIALOG_IMAGE_WIDTH, Constants.DIALOG_IMAGE_HEIGHT) : null;
				}

				@Override
				protected void done() {
					try {
						// Check if it's the right dialog
						if (audioFilesEditing.equals(files)) {
							ImageIcon cover = get();
							getEditTagDialog().getCover().setIcon(cover);
							getEditTagDialog().getCover().setEnabled(true);
							getEditTagDialog().getCoverButton().setEnabled(true);
							getEditTagDialog().getRemoveCoverButton().setEnabled(true);
						}
					} catch (InterruptedException e) {
						logger.error(LogCategories.FILE_READ, e);
					} catch (ExecutionException e) {
						logger.error(LogCategories.FILE_READ, e);
					}
				}
			}.execute();
		}

		Set<String> artists = new HashSet<String>();
		Set<String> albums = new HashSet<String>();
		Set<Integer> years = new HashSet<Integer>();
		Set<String> comments = new HashSet<String>();
		Set<String> genres = new HashSet<String>();
		Set<String> composers = new HashSet<String>();
		Set<String> lyrics = new HashSet<String>();
		Set<String> albumArtists = new HashSet<String>();
		for (int i = 0; i < files.size(); i++) {
			String artist = "";
			String album = "";
			int year = -1;
			String comment = "";
			String genre = "";
			String composer = "";
			String lyric = "";
			String albumArtist = "";
			if (files.get(i).getTag() != null) {
				Tag tag = files.get(i).getTag();
				artist = tag.getArtist();
				album = tag.getAlbum();
				year = tag.getYear();
				comment = tag.getComment();
				genre = tag.getGenre();
				composer = tag.getComposer();
				lyric = tag.getLyrics();
				albumArtist = tag.getAlbumArtist();
			}
			artists.add(artist);
			albums.add(album);
			years.add(year);
			comments.add(comment);
			genres.add(genre);
			composers.add(composer);
			lyrics.add(lyric);
			albumArtists.add(albumArtist);
		}

		if (artists.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getArtistTextField().getEditor().setItem(files.get(0).getTag().getArtist());
		} else {
			getDialogControlled().getArtistTextField().getEditor().setItem("");
		}

		if (albums.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getAlbumTextField().getEditor().setItem(files.get(0).getTag().getAlbum());
		} else {
			getDialogControlled().getAlbumTextField().getEditor().setItem("");
		}

		if (years.size() == 1 && files.get(0).getTag() != null && files.get(0).getTag().getYear() > 0) {
			getDialogControlled().getYearTextField().setText(Integer.toString(files.get(0).getTag().getYear()));
		} else {
			getDialogControlled().getYearTextField().setText("");
		}

		if (comments.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getCommentTextArea().setText(files.get(0).getTag().getComment());
			getDialogControlled().getCommentTextArea().setCaretPosition(0);
		} else {
			getDialogControlled().getCommentTextArea().setText("");
		}

		if (genres.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getGenreComboBox().getEditor().setItem(files.get(0).getTag().getGenre());
		} else {
			getDialogControlled().getGenreComboBox().getEditor().setItem("");
		}

		if (lyrics.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getLyricsTextArea().setText(files.get(0).getTag().getLyrics());
			getDialogControlled().getLyricsTextArea().setCaretPosition(0);
		} else {
			getDialogControlled().getLyricsTextArea().setText("");
		}

		if (composers.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getComposerTextField().setText(files.get(0).getTag().getComposer());
		} else {
			getDialogControlled().getComposerTextField().setText("");
		}

		if (albumArtists.size() == 1 && files.get(0).getTag() != null) {
			getDialogControlled().getAlbumArtistTextField().setText(files.get(0).getTag().getAlbumArtist());
		} else {
			getDialogControlled().getAlbumArtistTextField().setText("");
		}

		getDialogControlled().setVisible(true);
	}

	/**
	 * Returns edit tag dialog. Used for inner classes without increasing
	 * visibility of getDialogControlled
	 * 
	 * @return
	 */
	EditTagDialog getEditTagDialog() {
		return getDialogControlled();
	}

	/**
	 * Edits the tag.
	 */
	protected void editTag() {
		getLogger().debug(LogCategories.CONTROLLER);

		getDialogControlled().setVisible(false);

		// Build editor props
		Map<String, Object> properties = new HashMap<String, Object>();
		properties.put("TITLE", getDialogControlled().getTitleTextField().getText());
		if (getDialogControlled().getArtistTextField().getSelectedItem().toString().isEmpty()) {
			properties.put("ARTIST", null);
		} else {
			properties.put("ARTIST", getDialogControlled().getArtistTextField().getSelectedItem());
		}
		if (getDialogControlled().getAlbumTextField().getSelectedItem().toString().isEmpty()) {
			properties.put("ALBUM", null);
		} else {
			properties.put("ALBUM", getDialogControlled().getAlbumTextField().getSelectedItem());
		}
		if (getDialogControlled().getYearTextField().getText().isEmpty()) {
			properties.put("YEAR", null);
		} else {
			properties.put("YEAR", getDialogControlled().getYearTextField().getText());
		}
		if (getDialogControlled().getCommentTextArea().getText().isEmpty()) {
			properties.put("COMMENT", null);
		} else {
			properties.put("COMMENT", getDialogControlled().getCommentTextArea().getText());
		}
		if (getDialogControlled().getGenreComboBox().getSelectedIndex() == 0) {
			properties.put("GENRE", null);
		} else {
			properties.put("GENRE", getDialogControlled().getGenreComboBox().getSelectedItem());
		}
		if (getDialogControlled().getLyricsTextArea().getText().isEmpty()) {
			properties.put("LYRICS", null);
		} else {
			// Text area lina breaks are \n so in some OS (Windows) is not a correct line break -> Replace with OS line terminator
			String lyrics = getDialogControlled().getLyricsTextArea().getText();
			if (SystemProperties.OS.equals(OperatingSystem.WINDOWS)) {
				lyrics = lyrics.replaceAll("\n", "\r\n");
			}
			properties.put("LYRICS", lyrics);
		}
		if (getDialogControlled().getComposerTextField().getText().isEmpty()) {
			properties.put("COMPOSER", null);
		} else {
			properties.put("COMPOSER", getDialogControlled().getComposerTextField().getText());
		}
		if (getDialogControlled().getAlbumArtistTextField().getText().isEmpty()) {
			properties.put("ALBUM_ARTIST", null);
		} else {
			properties.put("ALBUM_ARTIST", getDialogControlled().getAlbumArtistTextField().getText());
		}

		properties.put("TRACK", getDialogControlled().getTrackNumberTextField().getText());

		properties.put("SHOULD_EDIT_COVER", shouldEditCover);
		properties.put("NEW_COVER", newCover);

		BackgroundExecutor.changeTags(audioFilesEditing, properties);
	}

	public boolean isShouldEditCover() {
		return shouldEditCover;
	}

	public void setShouldEditCover(boolean shouldEditCover) {
		this.shouldEditCover = shouldEditCover;
	}

	public byte[] getNewCover() {
		return newCover;
	}

	public void setNewCover(byte[] newCover) {
		this.newCover = newCover;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}
}
