/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingWorker;
import javax.swing.Timer;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.model.NavigationTableColumnModel;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.gui.views.dialogs.AlbumToolTip;
import net.sourceforge.atunes.gui.views.panels.NavigationPanel;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.model.PanelController;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationControllerState.ViewMode;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationMenusActionListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTabbedPaneChangeListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTableControlsActionListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTableMouseListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTreeControlsActionListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTreeControlsKeyListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTreeMouseListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTreeSelectionListener;
import net.sourceforge.atunes.kernel.controllers.navigation.listeners.NavigationTreeToolTipListener;
import net.sourceforge.atunes.kernel.handlers.DeviceHandler;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.handlers.PodcastFeedHandler;
import net.sourceforge.atunes.kernel.handlers.RadioHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler.SortType;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeed;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class NavigationController.
 * 
 * @author fleax
 */
public class NavigationController extends PanelController<NavigationPanel> {

	static Logger logger = new Logger();

	/** The state. */
	NavigationControllerState state;

	/** The node translator. */
	private NodeToSongsTranslator nodeTranslator;

	/** The current album tool tip content. */
	volatile Object currentAlbumToolTipContent;

	/** The album tool tip. */
	private AlbumToolTip albumToolTip;

	/** The timer. */
	private Timer timer = new Timer(0, new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			getAlbumToolTip().setVisible(true);

			final Album album = (Album) currentAlbumToolTipContent;
			SwingWorker<ImageIcon, Void> getAndSetImage = new SwingWorker<ImageIcon, Void>() {
				@Override
				protected ImageIcon doInBackground() throws Exception {
					return album.getPicture(Constants.TOOLTIP_IMAGE_WIDTH, Constants.TOOLTIP_IMAGE_HEIGHT);
				}

				@Override
				protected void done() {
					try {
						if (currentAlbumToolTipContent.equals(album)) {
							getAlbumToolTip().setPicture(get());
						}
					} catch (InterruptedException e) {
						logger.error(LogCategories.IMAGE, e);
					} catch (ExecutionException e) {
						logger.error(LogCategories.IMAGE, e);
					}
				}
			};
			getAndSetImage.execute();

		}
	});

	/**
	 * Instantiates a new navigation controller.
	 * 
	 * @param panel
	 *            the panel
	 */
	public NavigationController(NavigationPanel panel) {
		super(panel);
		state = new NavigationControllerState();
		nodeTranslator = new NodeToSongsTranslator(state);
		addBindings();
		addStateBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		// Keys
		getPanelControlled().getNonFavoriteAddToPlaylistMenuItem().setAccelerator(KeyStroke.getKeyStroke('+'));
		getPanelControlled().getNonFavoriteSetAsPlaylistMenuItem().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, ActionEvent.CTRL_MASK));
		// End keys

		getPanelControlled().getNavigationTable().setModel(new NavigationTableModel(this));
		getPanelControlled().getNavigationTable().setColumnModel(new NavigationTableColumnModel(this));
		//adjustColumnsWidth();

		NavigationTreeSelectionListener treeSelectionListener = new NavigationTreeSelectionListener(this, getPanelControlled());
		getPanelControlled().getRepositoryTree().addTreeSelectionListener(treeSelectionListener);
		getPanelControlled().getFavoritesTree().addTreeSelectionListener(treeSelectionListener);
		getPanelControlled().getDeviceTree().addTreeSelectionListener(treeSelectionListener);
		getPanelControlled().getRadioTree().addTreeSelectionListener(treeSelectionListener);
		getPanelControlled().getPodcastFeedTree().addTreeSelectionListener(treeSelectionListener);

		NavigationTreeMouseListener treeMouseListener = new NavigationTreeMouseListener(this, getPanelControlled());
		getPanelControlled().getRepositoryTree().addMouseListener(treeMouseListener);
		getPanelControlled().getFavoritesTree().addMouseListener(treeMouseListener);
		getPanelControlled().getDeviceTree().addMouseListener(treeMouseListener);
		getPanelControlled().getRadioTree().addMouseListener(treeMouseListener);
		getPanelControlled().getPodcastFeedTree().addMouseListener(treeMouseListener);

		NavigationTreeToolTipListener tooltipListener = new NavigationTreeToolTipListener(this, getPanelControlled());
		getPanelControlled().getRepositoryTree().addMouseMotionListener(tooltipListener);
		getPanelControlled().getRepositoryTreeScrollPane().addMouseWheelListener(tooltipListener);

		getPanelControlled().getNavigationTable().addMouseListener(new NavigationTableMouseListener(this, getPanelControlled()));
		getPanelControlled().getTabbedPane().addChangeListener(new NavigationTabbedPaneChangeListener(this, getPanelControlled()));

		NavigationTreeControlsActionListener treeControlsActionListener = new NavigationTreeControlsActionListener(this, getPanelControlled());
		getPanelControlled().getCollapseTree().addActionListener(treeControlsActionListener);
		getPanelControlled().getExpandTree().addActionListener(treeControlsActionListener);
		getPanelControlled().getFilterTextField().addActionListener(treeControlsActionListener);
		getPanelControlled().getClearFilterButton().addActionListener(treeControlsActionListener);
		getPanelControlled().getShowArtist().addActionListener(treeControlsActionListener);
		getPanelControlled().getShowAlbum().addActionListener(treeControlsActionListener);
		getPanelControlled().getShowGenre().addActionListener(treeControlsActionListener);
		getPanelControlled().getShowFolder().addActionListener(treeControlsActionListener);

		NavigationTreeControlsKeyListener treeControlsKeyListener = new NavigationTreeControlsKeyListener(this, getPanelControlled());
		getPanelControlled().getFilterTextField().addKeyListener(treeControlsKeyListener);

		NavigationTableControlsActionListener tableControlsActionListener = new NavigationTableControlsActionListener(this, getPanelControlled());
		getPanelControlled().getNavigationTableInfoButton().addActionListener(tableControlsActionListener);
		getPanelControlled().getSortByTrack().addActionListener(tableControlsActionListener);
		getPanelControlled().getSortByFile().addActionListener(tableControlsActionListener);
		getPanelControlled().getSortByTitle().addActionListener(tableControlsActionListener);
		getPanelControlled().getSortByModificationTime().addActionListener(tableControlsActionListener);

		NavigationMenusActionListener menusActionListener = new NavigationMenusActionListener(this, getPanelControlled());
		getPanelControlled().getAddToPlaylistMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getSetAsPlaylistMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoritePodcastFeedEntryDownload().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteRemoveOldPodcastFeedEntry().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteOpenFolderMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getPlayNowMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRemoveFromFavoritesMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteAddToPlaylistMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSetAsPlaylistMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoritePlayNowMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSetAsFavoriteSongMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSetAsFavoriteAlbumMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSetAsFavoriteArtistMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteEditTagMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteMarkPodcastEntryAsListened().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteEditTitlesMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteClearTagMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteExtractPictureMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getDeviceCopyToRepositoryMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getDeviceFillMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSearch().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteSearchAt().addActionListener(menusActionListener);
		getPanelControlled().getRadioAddRadioMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRadioAddFavoriteMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRadioRefreshMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRadioRenameRadioMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRadioRemoveRadioMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getRadioRenameLabelMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getPodcastFeedAddPodcastFeedMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getPodcastFeedRenamePodcastFeedMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getPodcastFeedRemovePodcastFeedMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getPodcastFeedMarkAsListenedMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteRemovePhysicallyMenuItem().addActionListener(menusActionListener);
		getPanelControlled().getNonFavoriteCopyToDeviceMenuItem().addActionListener(menusActionListener);
	}

	@Override
	protected NavigationPanel getPanelControlled() {
		return super.getPanelControlled();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Apply repository view state
		ViewMode viewMode = Kernel.getInstance().state.getRepositoryView();
		state.setViewMode(viewMode);
		if (viewMode == ViewMode.ARTIST) {
			getPanelControlled().getShowArtist().setSelected(true);
		} else if (viewMode == ViewMode.ALBUM) {
			getPanelControlled().getShowAlbum().setSelected(true);
		} else if (viewMode == ViewMode.GENRE) {
			getPanelControlled().getShowGenre().setSelected(true);
		} else if (viewMode == ViewMode.FOLDER) {
			getPanelControlled().getShowFolder().setSelected(true);
		}

		state.setSortType(SortType.BY_TRACK);
		getPanelControlled().getSortByTrack().setSelected(true);

		getPanelControlled().getNavigationTableInfoButton().setEnabled(false);
	}

	/**
	 * @return the timer
	 */
	public Timer getToolTipTimer() {
		return timer;
	}

	/**
	 * Gets the album tool tip.
	 * 
	 * @return the album tool tip
	 */
	public AlbumToolTip getAlbumToolTip() {
		if (albumToolTip == null) {
			JDialog.setDefaultLookAndFeelDecorated(false);
			albumToolTip = new AlbumToolTip();
			JDialog.setDefaultLookAndFeelDecorated(true);
		}
		return albumToolTip;
	}

	/**
	 * Gets the favorites tree model.
	 * 
	 * @return the favorites tree model
	 */
	public DefaultTreeModel getFavoritesTreeModel() {
		return getPanelControlled().getFavoritesTreeModel();
	}

	/**
	 * Get files of all selected elements in navigator.
	 * 
	 * @return the files selected in navigator
	 */
	public List<AudioFile> getFilesSelectedInNavigator() {
		List<AudioObject> files = new ArrayList<AudioObject>();
		if (getState().getPopupmenuCaller() == getPanelControlled().getNavigationTable()) {
			int[] rows = getPanelControlled().getNavigationTable().getSelectedRows();
			files.addAll(((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).getSongsAt(rows));
		} else if (getState().getPopupmenuCaller() == getPanelControlled().getRepositoryTree()) {
			TreePath[] paths = getPanelControlled().getRepositoryTree().getSelectionPaths();
			for (TreePath path : paths) {
				if (((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject() instanceof TreeObject) {
					files.addAll(((TreeObject) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject()).getAudioObjects());
				}
			}
		}
		return AudioFile.getAudioFiles(files);
	}

	/**
	 * Checks if a collection of files have the same parent file.
	 * 
	 * @param c
	 *            collection of files
	 * @return if a collection of files have the same parent file
	 */
	public boolean sameParentFile(Collection<? extends File> c) {
		Set<File> set = new HashSet<File>();
		for (File file : c) {
			set.add(file.getParentFile());
		}
		return set.size() == 1;
	}

	/**
	 * Gets the last album tool tip content.
	 * 
	 * @return the last album tool tip content
	 */
	public Object getCurrentAlbumToolTipContent() {
		return currentAlbumToolTipContent;
	}

	/**
	 * Gets the song in navigation table.
	 * 
	 * @param row
	 *            the row
	 * 
	 * @return the song in navigation table
	 */
	public AudioObject getSongInNavigationTable(int row) {
		return ((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).getSongAt(row);
	}

	/**
	 * Gets the songs for device tree node.
	 * 
	 * @param node
	 *            the node
	 * 
	 * @return the songs for device tree node
	 */
	public List<AudioObject> getSongsForDeviceTreeNode(DefaultMutableTreeNode node) {
		return nodeTranslator.getSongsForDeviceTreeNode(node);
	}

	/**
	 * Gets the songs for favorite tree node.
	 * 
	 * @param node
	 *            the node
	 * 
	 * @return the songs for favorite tree node
	 */
	public List<AudioObject> getSongsForFavoriteTreeNode(DefaultMutableTreeNode node) {
		return nodeTranslator.getSongsForFavoriteTreeNode(node);
	}

	/**
	 * Gets the songs for navigation tree.
	 * 
	 * @return the songs for navigation tree
	 */
	public List<AudioObject> getSongsForNavigationTree() {
		TreePath[] paths = getPanelControlled().getRepositoryTree().getSelectionPaths();
		return nodeTranslator.getSongsForNavigationTree(paths);
	}

	/**
	 * Gets the songs for podcast feed tree node.
	 * 
	 * @param node
	 *            the node
	 * 
	 * @return the songs for podcast feed tree node
	 */
	public List<AudioObject> getSongsForPodcastFeedTreeNode(DefaultMutableTreeNode node) {
		return nodeTranslator.getSongsForPodcastFeedTreeNode(node);
	}

	/**
	 * Gets the songs for radio tree node.
	 * 
	 * @param node
	 *            the node
	 * 
	 * @return the songs for radio tree node
	 */
	public List<AudioObject> getSongsForRadioTreeNode(DefaultMutableTreeNode node) {
		return nodeTranslator.getSongsForRadioTreeNode(node);
	}

	/**
	 * Gets the songs for tree node.
	 * 
	 * @param node
	 *            the node
	 * 
	 * @return the songs for tree node
	 */
	public List<AudioObject> getSongsForTreeNode(DefaultMutableTreeNode node) {
		return nodeTranslator.getSongsForTreeNode(node);
	}

	/**
	 * Gets the state.
	 * 
	 * @return the state
	 */
	public NavigationControllerState getState() {
		return state;
	}

	/**
	 * Notify device reload.
	 */
	public void notifyDeviceReload() {
		refreshDeviceTreeContent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	public void notifyReload() {
		refreshTreeContent();
		refreshFavoriteTree();
	}

	/**
	 * Refresh device tree content.
	 */
	public void refreshDeviceTreeContent() {
		getLogger().debug(LogCategories.CONTROLLER);

		Map<String, ?> data;
		if (state.getViewMode() == ViewMode.GENRE) {
			data = DeviceHandler.getInstance().getDeviceGenreStructure();
			//TagViewRefresher.refreshGenreTree(data, getPanelControlled().getRepositoryTree(), getPanelControlled().getRepositoryTreeModel(), state.getCurrentFilter());
		} else if (state.getViewMode() == ViewMode.FOLDER) {
			data = DeviceHandler.getInstance().getDeviceFolderStructure();
		} else {
			data = DeviceHandler.getInstance().getDeviceArtistAndAlbumStructure();
		}

		DeviceViewRefresher.refresh(data, getPanelControlled().getDeviceTree(), getPanelControlled().getDeviceTreeModel(), state.getViewMode(), state.getCurrentFilter());
		getPanelControlled().getDeviceTree().expandRow(0);
		getPanelControlled().getDeviceTree().setSelectionRow(0);
	}

	/**
	 * Refresh favorite tree.
	 */
	public void refreshFavoriteTree() {
		getLogger().debug(LogCategories.CONTROLLER);

		int[] rowSelected = getPanelControlled().getFavoritesTree().getSelectionRows();

		Map<String, Artist> artists = FavoritesHandler.getInstance().getFavoriteArtistsInfo();
		Map<String, Album> albums = FavoritesHandler.getInstance().getFavoriteAlbumsInfo();
		FavoriteViewRefresher.refresh(getPanelControlled().getFavoritesTree(), artists, albums, getPanelControlled().getFavoritesTreeModel());
		getPanelControlled().getFavoritesTree().expandRow(0);
		getPanelControlled().getFavoritesTree().setSelectionRow(rowSelected != null ? rowSelected[0] : 0);
	}

	/**
	 * Refresh podcast feed tree content.
	 */
	public void refreshPodcastFeedTreeContent() {
		getLogger().debug(LogCategories.CONTROLLER);
		List<PodcastFeed> podcastFeeds = PodcastFeedHandler.getInstance().getPodcastFeeds();

		// remember selected rows (if no podcast feed was added or removed)
		int[] selectedRows = getPanelControlled().getPodcastFeedTree().getSelectionRows();
		int podcastCount = podcastFeeds.size();
		int podcastInTreeModelCount = getPanelControlled().getPodcastFeedTreeModel().getChildCount(getPanelControlled().getPodcastFeedTreeModel().getRoot());
		if (podcastCount != podcastInTreeModelCount) {
			selectedRows = null;
		}
		PodcastFeedViewRefresher.refresh(podcastFeeds, getPanelControlled().getPodcastFeedTreeModel(), state.getCurrentFilter());
		if (selectedRows != null) {
			getPanelControlled().getPodcastFeedTree().setSelectionRows(selectedRows);
		} else {
			getPanelControlled().getPodcastFeedTree().setSelectionRow(0);
		}

		getPanelControlled().getPodcastFeedTree().expandRow(0);
	}

	/**
	 * Refreshes the radio view.
	 */
	public void refreshRadioTreeContent() {
		getLogger().debug(LogCategories.CONTROLLER);
		boolean showAllStations = Kernel.getInstance().state.isShowAllRadioStations();
		List<Radio> radios = RadioHandler.getInstance().getRadios();
		List<Radio> presetRadios = RadioHandler.getInstance().getRadioPresets();
		RadioViewRefresher.refresh(radios, presetRadios, getPanelControlled().getRadioTreeModel(), state.getCurrentFilter(), showAllStations);
		getPanelControlled().getRadioTree().expandRow(0);
		getPanelControlled().getRadioTree().setSelectionRow(0);
	}

	/**
	 * Refresh table.
	 */
	public void refreshTable() {
		((NavigationTableModel) getPanelControlled().getNavigationTable().getModel()).refresh();
	}

	/**
	 * Refresh repository view tree content.
	 */
	public void refreshRepositoryViewTreeContent() {
		getLogger().debug(LogCategories.CONTROLLER);

		if (state.getViewMode() == ViewMode.GENRE) {
			Map<String, Genre> data = RepositoryHandler.getInstance().getGenreStructure();
			RepositoryViewRefresher.refreshGenreTree(data, getPanelControlled().getRepositoryTree(), getPanelControlled().getRepositoryTreeModel(), state.getCurrentFilter());
		} else if (state.getViewMode() == ViewMode.FOLDER) {
			Map<String, Folder> data = RepositoryHandler.getInstance().getFolderStructure();
			RepositoryViewRefresher.refreshFolderView(data, getPanelControlled().getRepositoryTree(), getPanelControlled().getRepositoryTreeModel(), state.getCurrentFilter());
		} else {
			Map<String, Artist> data = RepositoryHandler.getInstance().getArtistAndAlbumStructure();
			RepositoryViewRefresher.refreshTagView(data, getPanelControlled().getRepositoryTree(), getPanelControlled().getRepositoryTreeModel(), state.getViewMode(), state
					.getCurrentFilter());
		}
		getPanelControlled().getRepositoryTree().expandRow(0);
		// If view is folder, then expand root child nodes too
		if (state.getViewMode() == ViewMode.FOLDER) {
			// Expand root childs
			TreePath rootPath = getPanelControlled().getRepositoryTree().getPathForRow(0);
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) rootPath.getLastPathComponent();
			for (int i = 0; i < node.getChildCount(); i++) {
				TreeNode childNode = node.getChildAt(i);
				TreePath childPath = rootPath.pathByAddingChild(childNode);
				getPanelControlled().getRepositoryTree().expandPath(childPath);
			}
			getPanelControlled().getRepositoryTree().setSelectionRow(0);
		}
	}

	/**
	 * Refresh tree content.
	 */
	public void refreshTreeContent() {
		refreshRepositoryViewTreeContent();
		refreshDeviceTreeContent();
		refreshRadioTreeContent();
		refreshPodcastFeedTreeContent();
	}

	/**
	 * Sets the current album tool tip content.
	 * 
	 * @param currentAlbumToolTipContent
	 *            the new current album tool tip content
	 */
	public void setCurrentAlbumToolTipContent(Object currentAlbumToolTipContent) {
		this.currentAlbumToolTipContent = currentAlbumToolTipContent;
	}

	/**
	 * Sets the navigation view.
	 * 
	 * @param view
	 *            the new navigation view
	 */
	public void setNavigationView(int view) {
		getLogger().debug(LogCategories.CONTROLLER, new String[] { Integer.toString(view) });
		state.setNavigationView(view);
		Kernel.getInstance().state.setNavigationView(view);
		getPanelControlled().getTabbedPane().setSelectedIndex(view);

		getPanelControlled().getShowAlbum().setEnabled(view == NavigationControllerViews.REPOSITORY_VIEW || view == NavigationControllerViews.DEVICE_VIEW);
		getPanelControlled().getShowArtist().setEnabled(view == NavigationControllerViews.REPOSITORY_VIEW || view == NavigationControllerViews.DEVICE_VIEW);
		getPanelControlled().getShowFolder().setEnabled(view == NavigationControllerViews.REPOSITORY_VIEW || view == NavigationControllerViews.DEVICE_VIEW);
		getPanelControlled().getShowGenre().setEnabled(view == NavigationControllerViews.REPOSITORY_VIEW || view == NavigationControllerViews.DEVICE_VIEW);

		if (view == NavigationControllerViews.FAVORITE_VIEW) {
			getPanelControlled().getFilterLabel().setEnabled(false);
			getPanelControlled().getFilterTextField().setEnabled(false);
			getPanelControlled().getClearFilterButton().setEnabled(false);
		} else {
			getPanelControlled().getFilterLabel().setEnabled(true);
			getPanelControlled().getFilterTextField().setEnabled(true);
			getPanelControlled().getClearFilterButton().setEnabled(true);
		}
	}

	/**
	 * Sort.
	 * 
	 * @param songs
	 *            the songs
	 * @param type
	 *            the type
	 * 
	 * @return the list< audio object>
	 */
	public List<AudioObject> sort(List<AudioObject> songs, SortType type) {
		return RepositoryHandler.getInstance().sort(songs, type);
	}

	/**
	 * Returns navigator device tree
	 * 
	 * @return
	 */
	public JTree getDeviceTree() {
		return getPanelControlled().getDeviceTree();
	}

	/**
	 * Returns navigator device tree scroll pane
	 * 
	 * @return
	 */
	public JScrollPane getDeviceTreeScrollPane() {
		return getPanelControlled().getDeviceTreeScrollPane();
	}

}
