/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationControllerState.ViewMode;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class TagViewRefresher.
 */
public class RepositoryViewRefresher {

	// Ignores "The "
	/** The smart comparator. */
	private static Comparator<String> smartComparator = new Comparator<String>() {

		private String removeThe(String str) {
			if (str.toLowerCase().startsWith("the ") && str.length() > 4) {
				return str.substring(4);
			}
			return str;
		}

		@Override
		public int compare(String s1, String s2) {
			return removeThe(s1).compareToIgnoreCase(removeThe(s2));
		}
	};

	/** The comparator. */
	private static Comparator<String> comparator = new Comparator<String>() {
		@Override
		public int compare(String s1, String s2) {
			return s1.compareToIgnoreCase(s2);
		}
	};

	/**
	 * Refresh.
	 * 
	 * @param structure
	 *            the structure
	 * @param tree
	 *            the tree
	 * @param treeModel
	 *            the tree model
	 * @param viewMode
	 *            the view mode
	 * @param currentFilter
	 *            the current filter
	 */
	public static void refreshTagView(Map<String, Artist> structure, JTree tree, DefaultTreeModel treeModel, ViewMode viewMode, String currentFilter) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();

		// Get objects selected before refreshing tree
		Set<TreeObject> objectsSelected = new HashSet<TreeObject>();
		List<DefaultMutableTreeNode> nodesToSelect = new ArrayList<DefaultMutableTreeNode>();
		TreePath[] pathsSelected = tree.getSelectionPaths();

		// If any node was selected
		if (pathsSelected != null) {
			for (TreePath pathSelected : pathsSelected) {
				Object obj = ((DefaultMutableTreeNode) pathSelected.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsSelected.add((TreeObject) obj);
				}
			}
		}

		// Get objects expanded before refreshing tree
		Set<TreeObject> objectsExpanded = new HashSet<TreeObject>();
		List<DefaultMutableTreeNode> nodesToExpand = new ArrayList<DefaultMutableTreeNode>();
		Enumeration<TreePath> enume = tree.getExpandedDescendants(new TreePath(root.getPath()));

		// If any node was expanded
		if (enume != null) {
			while (enume.hasMoreElements()) {
				TreePath p = enume.nextElement();
				Object obj = ((DefaultMutableTreeNode) p.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsExpanded.add((TreeObject) obj);
				}
			}
		}
		root.setUserObject(LanguageTool.getString("REPOSITORY"));
		root.removeAllChildren();
		List<String> artistNamesList = new ArrayList<String>(structure.keySet());
		if (Kernel.getInstance().state.isUseSmartTagViewSorting()) {
			Collections.sort(artistNamesList, smartComparator);
		} else {
			Collections.sort(artistNamesList, comparator);
		}

		if (viewMode == ViewMode.ARTIST) {
			for (int i = 0; i < artistNamesList.size(); i++) {
				Artist artist = structure.get(artistNamesList.get(i));
				DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
				List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
				if (Kernel.getInstance().state.isUseSmartTagViewSorting()) {
					Collections.sort(albumNamesList, smartComparator);
				} else {
					Collections.sort(albumNamesList, comparator);
				}
				if (currentFilter == null || artist.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
					for (int j = 0; j < albumNamesList.size(); j++) {
						Album album = artist.getAlbum(albumNamesList.get(j));
						DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
						artistNode.add(albumNode);
					}
					root.add(artistNode);
					// Reload causes very important lag on large collections and if it is not used
					// selection does not work.

					// If node was selected before refreshing...
					if (objectsSelected.contains(artistNode.getUserObject())) {
						nodesToSelect.add(artistNode);
					}
					// If node was expanded before refreshing...
					if (objectsExpanded.contains(artistNode.getUserObject())) {
						nodesToExpand.add(artistNode);
					}
				}
			}
			treeModel.reload();
			for (DefaultMutableTreeNode node : nodesToExpand) {
				tree.expandPath(new TreePath(node.getPath()));
			}
			// Once tree has been refreshed, select previously selected nodes
			if (nodesToSelect.isEmpty()) {
				tree.setSelectionRow(0);
			} else {
				TreePath[] pathsToSelect = new TreePath[nodesToSelect.size()];
				int i = 0;
				for (DefaultMutableTreeNode node : nodesToSelect) {
					pathsToSelect[i++] = new TreePath(node.getPath());
				}
				tree.setSelectionPaths(pathsToSelect);
			}
		} else {
			// Album view

			// Gather albums
			Map<String, Album> albums = new HashMap<String, Album>();
			DefaultMutableTreeNode albumNode = null;
			for (int i = 0; i < artistNamesList.size(); i++) {
				Artist artist = structure.get(artistNamesList.get(i));
				List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
				Collections.sort(albumNamesList);
				for (int j = 0; j < albumNamesList.size(); j++) {
					String a = albumNamesList.get(j);
					Album album = artist.getAlbum(a);
					if (currentFilter == null || album.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
						albums.put(album.getName() + artist.toString(), album);
					}
				}
			}

			// Then add them in the tree with the artists
			List<String> albumsList = new ArrayList<String>(albums.keySet());
			Collections.sort(albumsList, comparator);
			for (int i = 0; i < albumsList.size(); i++) {
				Album a = albums.get(albumsList.get(i));
				DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(a.getArtist());
				if (i == 0 || !albums.get(albumsList.get(i)).toString().equals((albums.get(albumsList.get(i - 1))).toString())) {
					albumNode = new DefaultMutableTreeNode(a);
					root.add(albumNode);
					// If node was selected before refreshing...
					if (objectsSelected.contains(albumNode.getUserObject())) {
						nodesToSelect.add(albumNode);
					}
				}
				if (albumNode != null) {
					albumNode.add(artistNode);
				}
				// That one is a workaround because the Artist class returns all audiofiles of an artist and not only 
				// the songs from a specific album, so we add that album again. If you have a better idea you can improve
				DefaultMutableTreeNode subAlbumNode = new DefaultMutableTreeNode(a.getArtist().getAlbum(a.getName()));
				artistNode.add(subAlbumNode);
			}

			// Reload the tree to refresh content
			treeModel.reload();

			// Once tree has been refreshed, expand previously expanded nodes
			for (DefaultMutableTreeNode node : nodesToExpand) {
				tree.expandPath(new TreePath(node.getPath()));
			}

			// Once tree has been refreshed, select previously selected nodes
			if (nodesToSelect.isEmpty()) {
				tree.setSelectionRow(0);
			} else {
				TreePath[] pathsToSelect = new TreePath[nodesToSelect.size()];
				int i = 0;
				for (DefaultMutableTreeNode node : nodesToSelect) {
					pathsToSelect[i++] = new TreePath(node.getPath());
				}
				tree.setSelectionPaths(pathsToSelect);
			}
		}
	}

	/**
	 * Refresh genre tree.
	 * 
	 * @param structure
	 *            the structure
	 * @param tree
	 *            the tree
	 * @param treeModel
	 *            the tree model
	 * @param currentFilter
	 *            the current filter
	 */
	public static void refreshGenreTree(Map<String, Genre> structure, JTree tree, DefaultTreeModel treeModel, String currentFilter) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();

		// Get objects selected before refreshing tree
		Set<TreeObject> objectsSelected = new HashSet<TreeObject>();
		List<DefaultMutableTreeNode> nodesToSelect = new ArrayList<DefaultMutableTreeNode>();
		TreePath[] pathsSelected = tree.getSelectionPaths();

		// If any node was selected
		if (pathsSelected != null) {
			for (TreePath pathSelected : pathsSelected) {
				Object obj = ((DefaultMutableTreeNode) pathSelected.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsSelected.add((TreeObject) obj);
				}
			}
		}

		// Get objects expanded before refreshing tree
		Set<TreeObject> objectsExpanded = new HashSet<TreeObject>();
		List<DefaultMutableTreeNode> nodesToExpand = new ArrayList<DefaultMutableTreeNode>();
		Enumeration<TreePath> enume = tree.getExpandedDescendants(new TreePath(root.getPath()));

		// If any node was expanded
		if (enume != null) {
			while (enume.hasMoreElements()) {
				TreePath p = enume.nextElement();
				Object obj = ((DefaultMutableTreeNode) p.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsExpanded.add((TreeObject) obj);
				}
			}
		}

		root.removeAllChildren();
		List<String> genreNamesList = new ArrayList<String>(structure.keySet());
		Collections.sort(genreNamesList, comparator);

		for (int i = 0; i < genreNamesList.size(); i++) {
			Genre genre = structure.get(genreNamesList.get(i));
			if (currentFilter == null || genre.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
				DefaultMutableTreeNode genreNode = new DefaultMutableTreeNode(genre);
				// If node was selected before refreshing...
				if (objectsSelected.contains(genreNode.getUserObject())) {
					nodesToSelect.add(genreNode);
				}
				// If node was expanded before refreshing...
				if (objectsExpanded.contains(genreNode.getUserObject())) {
					nodesToExpand.add(genreNode);
				}
				List<String> artistNamesList = new ArrayList<String>(genre.getArtists().keySet());
				Collections.sort(artistNamesList);
				for (int j = 0; j < artistNamesList.size(); j++) {
					Artist artist = genre.getArtist(artistNamesList.get(j));
					DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
					List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
					Collections.sort(albumNamesList);
					for (int k = 0; k < albumNamesList.size(); k++) {
						Album album = artist.getAlbum(albumNamesList.get(k));
						DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
						artistNode.add(albumNode);
						genreNode.add(artistNode);
						// If node was selected before refreshing...
						if (objectsSelected.contains(artistNode.getUserObject())) {
							nodesToSelect.add(artistNode);
						}
						// If node was expanded before refreshing...
						if (objectsExpanded.contains(artistNode.getUserObject()) && objectsExpanded.contains(((DefaultMutableTreeNode) artistNode.getParent()).getUserObject())) {
							nodesToExpand.add(artistNode);
						}
					}
				}
				root.add(genreNode);
			}
		}

		treeModel.reload();
		// Once tree has been refreshed, expand previously expanded nodes
		for (DefaultMutableTreeNode node : nodesToExpand) {
			tree.expandPath(new TreePath(node.getPath()));
		}

		// Once tree has been refreshed, select previously selected nodes
		if (nodesToSelect.isEmpty()) {
			tree.setSelectionRow(0);
		} else {
			TreePath[] pathsToSelect = new TreePath[nodesToSelect.size()];
			int i = 0;
			for (DefaultMutableTreeNode node : nodesToSelect) {
				pathsToSelect[i++] = new TreePath(node.getPath());
			}
			tree.setSelectionPaths(pathsToSelect);
		}

	}

	/**
	 * Refresh folder view
	 * 
	 * @param structure
	 *            the structure
	 * @param tree
	 *            the tree
	 * @param treeModel
	 *            the tree model
	 * @param currentFilter
	 *            the current filter
	 */
	public static void refreshFolderView(Map<String, Folder> structure, JTree tree, DefaultTreeModel treeModel, String currentFilter) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();

		// Get objects selected before refreshing tree
		Set<TreeObject> objectsSelected = new HashSet<TreeObject>();
		TreePath[] pathsSelected = tree.getSelectionPaths();

		// If any node was selected
		if (pathsSelected != null) {
			for (TreePath pathSelected : pathsSelected) {
				Object obj = ((DefaultMutableTreeNode) pathSelected.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsSelected.add((TreeObject) obj);
				}
			}
		}

		// Get objects expanded before refreshing tree
		Set<TreeObject> objectsExpanded = new HashSet<TreeObject>();
		Enumeration<TreePath> enume = tree.getExpandedDescendants(new TreePath(root.getPath()));

		// If any node was expanded
		if (enume != null) {
			while (enume.hasMoreElements()) {
				TreePath p = enume.nextElement();
				Object obj = ((DefaultMutableTreeNode) p.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsExpanded.add((TreeObject) obj);
				}
			}
		}

		// Refresh nodes
		root.setUserObject(LanguageTool.getString("FOLDERS"));
		root.removeAllChildren();
		RefreshUtils.addFolderNodes(structure, root, currentFilter);

		treeModel.reload();

		// Get nodes to select after refresh
		List<DefaultMutableTreeNode> nodesToSelect = getNodes(root, objectsSelected);

		// Get nodes to expand after refresh
		List<DefaultMutableTreeNode> nodesToExpand = getNodes(root, objectsExpanded);

		// Once tree has been refreshed, expand previously expanded nodes
		for (DefaultMutableTreeNode node : nodesToExpand) {
			tree.expandPath(new TreePath(node.getPath()));
		}

		// Once tree has been refreshed, select previously selected nodes
		if (nodesToSelect.isEmpty()) {
			tree.setSelectionRow(0);
		} else {
			TreePath[] pathsToSelect = new TreePath[nodesToSelect.size()];
			int i = 0;
			for (DefaultMutableTreeNode node : nodesToSelect) {
				pathsToSelect[i++] = new TreePath(node.getPath());
			}
			tree.setSelectionPaths(pathsToSelect);
		}

	}

	/**
	 * Recursive method to find nodes, based on whether user objects are present
	 * in given set
	 * 
	 * @param rootNode
	 * @param userObjects
	 * @return list of nodes
	 */
	private static List<DefaultMutableTreeNode> getNodes(DefaultMutableTreeNode rootNode, Set<TreeObject> userObjects) {
		List<DefaultMutableTreeNode> result = new ArrayList<DefaultMutableTreeNode>();
		if (userObjects.contains(rootNode.getUserObject())) {
			result.add(rootNode);
		}
		for (int i = 0; i < rootNode.getChildCount(); i++) {
			result.addAll(getNodes((DefaultMutableTreeNode) rootNode.getChildAt(i), userObjects));
		}
		return result;
	}
}
