/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.util.ArrayList;
import java.util.List;

import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.modules.playlist.ListOfPlayLists;
import net.sourceforge.atunes.kernel.modules.playlist.PlayList;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListCommonOps;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListFilterOps;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * The Class MultiplePlaylistHandler.
 */
public class MultiplePlaylistHandler {

	/** The instance. */
	private static MultiplePlaylistHandler instance;

	/** The play list counter. */
	private static int playListCounter = 1;

	/** The play lists. */
	private List<PlayList> playLists;

	/** The current play list index. */
	private int currentPlayListIndex = 0;

	/**
	 * Instantiates a new multiple playlist handler.
	 */
	private MultiplePlaylistHandler() {
		playLists = new ArrayList<PlayList>();
	}

	/**
	 * Gets the single instance of MultiplePlaylistHandler.
	 * 
	 * @return single instance of MultiplePlaylistHandler
	 */
	public static MultiplePlaylistHandler getInstance() {
		if (instance == null) {
			instance = new MultiplePlaylistHandler();
		}
		return instance;
	}

	/**
	 * Adds a playlist.
	 * 
	 * @param list
	 *            the list
	 */
	public void addListOfPlayLists(ListOfPlayLists list) {
		for (PlayList playlist : list.getPlayLists()) {
			playLists.add(playlist);
			ControllerProxy.getInstance().getPlayListTabController().newPlayList(getNameForPlaylist(playlist));
		}
		currentPlayListIndex = list.getSelectedPlayList();
		ControllerProxy.getInstance().getPlayListTabController().forceSwitchTo(currentPlayListIndex);

		setPlayList(playLists.get(currentPlayListIndex));
	}

	/**
	 * Check if any of audio objects are in any of the playlists.
	 * 
	 * @param audioObjects
	 *            the audio objects
	 * 
	 * @return true, if check if play lists contains audio objects
	 */
	public boolean checkIfPlayListsContainsAudioObjects(List<AudioFile> audioObjects) {
		for (PlayList pl : playLists) {
			for (AudioFile af : audioObjects) {
				if (pl.contains(af)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Deletes play list referred by index.
	 * 
	 * @param index
	 *            the index
	 */
	public void deletePlayList(int index) {
		// if there is only one play list, don't delete
		if (playLists.size() <= 1) {
			return;
		}
		// delete tab and playlist
		// NOTE: deleting current tab play list calls automatically to switchToPlayList method
		if (index != currentPlayListIndex) {
			// Remove playlist from list
			playLists.remove(index);
			// If index < currentPlayListIndex, current play list has been moved to left, so
			// decrease in 1
			if (index < currentPlayListIndex) {
				currentPlayListIndex--;
			}
			// Delete tab
			ControllerProxy.getInstance().getPlayListTabController().deletePlayList(index);
		} else {
			// index == currentPlayList
			// switch play list and then delete
			if (index == 0) {
				switchToPlaylist(1);
			} else {
				switchToPlaylist(currentPlayListIndex - 1);
			}
			deletePlayList(index);
		}
	}

	/**
	 * Gets the list of play lists.
	 * 
	 * @return the list of play lists
	 */
	public ListOfPlayLists getListOfPlayLists() {
		ListOfPlayLists l = new ListOfPlayLists();

		// Clone play lists to make changes in returned list if current play list is filtered
		l.setPlayLists(new ArrayList<PlayList>(playLists));
		l.setSelectedPlayList(currentPlayListIndex);

		// If current play list is filtered return non-filtered play list
		if (PlayListFilterOps.isFiltered()) {
			l.getPlayLists().remove(currentPlayListIndex);
			l.getPlayLists().add(currentPlayListIndex, PlayListFilterOps.getNonFilteredPlayList());
		}

		return l;
	}

	/**
	 * Gets the name for playlist.
	 * 
	 * @param pl
	 *            the pl
	 * 
	 * @return the name for playlist
	 */

	private String getNameForPlaylist(PlayList pl) {
		if (pl == null || pl.getName() == null) {
			return StringUtils.getString(LanguageTool.getString("PLAYLIST"), " ", playListCounter++);
		}

		return pl.getName();
	}

	/**
	 * Called to create a new play list with given audio objects.
	 * 
	 * @param audioObjects
	 *            the audio objects
	 */
	public void newPlayList(List<AudioObject> audioObjects) {
		String nameOfNewPlayList = getNameForPlaylist(null);
		PlayList newPlayList;
		if (audioObjects == null) {
			newPlayList = new PlayList();
		} else {
			newPlayList = new PlayList(audioObjects);
		}
		newPlayList.setName(nameOfNewPlayList);
		playLists.add(newPlayList);

		ControllerProxy.getInstance().getPlayListTabController().newPlayList(nameOfNewPlayList);
	}

	/**
	 * Called to create a new play list with given audio objects.
	 * 
	 * @param nameOfNewPlayList
	 *            the name of the new play list as shown in play list tab
	 * 
	 * @param audioObjects
	 *            the audio objects to add to the new play list
	 */
	public void newPlayList(String nameOfNewPlayList, List<AudioObject> audioObjects) {
		PlayList newPlayList;
		if (audioObjects == null) {
			newPlayList = new PlayList();
		} else {
			newPlayList = new PlayList(audioObjects);
		}
		newPlayList.setName(nameOfNewPlayList);
		playLists.add(newPlayList);

		ControllerProxy.getInstance().getPlayListTabController().newPlayList(nameOfNewPlayList);
	}

	/**
	 * Removes songs from other playlists than current one.
	 * 
	 * @param audioFiles
	 *            the audio files
	 * 
	 * @return true if any song has been removed
	 */
	public boolean removeSongsFromOtherPlayLists(List<AudioFile> audioFiles) {
		boolean removed = false;
		for (PlayList pl : playLists) {
			if (playLists.indexOf(pl) != currentPlayListIndex) {
				pl.removeAll(audioFiles);
				removed = true;
			}
		}

		return removed;
	}

	/**
	 * Renames a play list.
	 * 
	 * @param index
	 *            the index
	 * @param newName
	 *            the new name
	 */
	public void renamePlayList(int index, String newName) {
		playLists.get(index).setName(newName);
		ControllerProxy.getInstance().getPlayListTabController().renamePlayList(index, newName);
	}

	/**
	 * Called when switching play list at tabbed pane.
	 * 
	 * @param index
	 *            the index
	 */
	public void switchToPlaylist(int index) {
		// If play list is the same, do nothing, except if this method is called when deleting a play list
		if (index == currentPlayListIndex) {
			return;
		}

		PlayerHandler.getInstance().stopCurrentAudioObject(false);

		// Remove filter from current play list before change play list
		PlayListHandler.getInstance().setFilter(null);

		currentPlayListIndex = index;

		PlayList newSelectedPlayList = playLists.get(index);

		// Remove songs from model
		VisualHandler.getInstance().getPlayListTableModel().removeSongs();

		// Set selection interval to none
		VisualHandler.getInstance().getPlayListTable().getSelectionModel().setSelectionInterval(-1, -1);

		setPlayList(newSelectedPlayList);
		PlayListCommonOps.refreshPlayList();
	}

	/**
	 * Sets the play list.
	 * 
	 * @param playList
	 *            the new play list
	 */
	protected static void setPlayList(PlayList playList) {
		ControllerProxy.getInstance().getPlayListController().notifyAudioObjectsAddedToController(playList, playList.getCurrentAudioObjectIndex());
		VisualHandler.getInstance().showPlayListInformation(playList);
		PlayListHandler.getInstance().selectedAudioObjectChanged(playList.getCurrentAudioObject());
	}

	/**
	 * Gets the current play list index.
	 * 
	 * @return the currentPlayListIndex
	 */
	public int getCurrentPlayListIndex() {
		return currentPlayListIndex;
	}

	public PlayList getCurrentPlayList() {
		return playLists.get(currentPlayListIndex);
	}

	public void setCurrentPlayList(PlayList playList) {
		playLists.remove(currentPlayListIndex);
		playLists.add(currentPlayListIndex, playList);
	}
}
