/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.PropertyResourceBundle;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable.PlayState;
import net.sourceforge.atunes.kernel.ApplicationFinishListener;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.playList.PlayListController;
import net.sourceforge.atunes.kernel.modules.player.PlayerEngineCapability;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.TempFolder;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class PlayerHandler. This class has common logic for all player engines
 */
public abstract class PlayerHandler implements ApplicationFinishListener {

	//////////////////////////////////////////// METHODS TO IMPLEMENT BY ENGINES ////////////////////////////////////////////

	/**
	 * Checks if engine is currently playing (<code>true</code>) or not (
	 * <code>false</code>)
	 * 
	 * @return <code>true</code> if engine is currently playing
	 */
	public abstract boolean isEnginePlaying();

	/**
	 * This method must be implemented by player engines. This method must check
	 * system to determine if player engine is available (check for libraries or
	 * commands)
	 * 
	 * @return <code>true</code> if engine is available in the system and can be
	 *         used to play, <code>false</code> otherwise
	 */
	protected abstract boolean isEngineAvailable();

	/**
	 * play this audio object
	 * 
	 * @param audioObjectToPlay
	 *            audio object to play. May be cashed to temp dirs or the same
	 *            as audioObject.
	 * @param audioObject
	 *            original audio object to update statistics TODO: Change method
	 *            signature and add javadoc
	 */
	protected abstract void startPlayback(AudioObject audioObjectToPlay, AudioObject audioObject);

	/**
	 * This method must be implemented by player engines. This method pauses
	 * playback of current audio object without stopping it. Resuming after this
	 * called should continue playback from the position when paused
	 */
	protected abstract void pausePlayback();

	/**
	 * This method must be implemented by player engines. This method resumes
	 * playback of current audio object previously paused. Call this method
	 * should continue playback from the position when paused
	 */
	protected abstract void resumePlayback();

	/**
	 * This method must be implemented by player engines. Stop playing current
	 * song
	 * 
	 * @param userStopped
	 *            {@code true} if stopped by user input, {@code false}
	 *            otherwise.
	 * @param useFadeAway
	 *            if {@code true} - fade away then stop. Stop immediately
	 *            otherwise.
	 */
	protected abstract void stopPlayback(boolean userStopped, boolean useFadeAway);

	/**
	 * This method must be implemented by player engines. Applies a seek
	 * operation in player engine
	 * 
	 * @param perCent
	 *            0-100
	 */
	protected abstract void seekTo(double perCent);

	/**
	 * This method must be implemented by player engines. Applies volume value
	 * in player engine
	 * 
	 * @param perCent
	 *            0-100
	 */
	public abstract void setVolume(int perCent);

	/**
	 * This method must be implemented by player engines. Apply mute state in
	 * player engine
	 * 
	 * @param state
	 *            : enabled (<code>true</code>) or disabled (<code>false</code>)
	 * 
	 */
	public abstract void applyMuteState(boolean state);

	/**
	 * This method must be implemented by player engines. Returns if engine
	 * supports capability or not
	 * 
	 * @return <code>true</code> if engine supports capability or not
	 */
	public abstract boolean supportsCapability(PlayerEngineCapability capability);

	/**
	 * This method must be implemented by player engines. Method to apply
	 * equalizer values in player engine
	 * 
	 * @param values
	 */
	protected abstract void applyEqualization(float[] values);

	/**
	 * This method must be implemented by player engines. Transform values
	 * retrieved from equalizer dialog to values for player engine
	 * 
	 * @param values
	 * @return
	 */
	protected abstract float[] transformEqualizerValues(float[] values);

	/**
	 * This method must be implemented by player engines It's called when
	 * application finishes
	 */
	protected abstract void finishPlayer();

	///////////////////////////////////////// END OF METHODS TO IMPLEMENT BY ENGINES ////////////////////////////////////////

	/**
	 * Gets the single instance of PlayerHandler. This method returns player
	 * engine configured by user or default if it's available on the system
	 * 
	 * @return single instance of PlayerHandler
	 */
	public static final synchronized PlayerHandler getInstance() {
		if (instance == null) {
			// Get engines list
			Map<String, PlayerHandler> engines = getEnginesFromConfig();

			// Remove unsupported engines
			Iterator<PlayerHandler> it = engines.values().iterator();
			while (it.hasNext()) {
				if (!it.next().isEngineAvailable()) {
					it.remove();
				}
			}

			// Update engine names
			engineNames = engines.keySet().toArray(new String[] {});
			Arrays.sort(engineNames);

			if (engines.isEmpty()) {
				notifyPlayerError(new IllegalStateException(LanguageTool.getString("NO_PLAYER_ENGINE")));
			} else {
				// Get engine of application state (default or selected by user)
				String selectedEngine = Kernel.getInstance().state.getPlayerEngine();

				// If selected engine is not available then try default engine or another one
				if (!engines.containsKey(selectedEngine)) {
					if (engines.containsKey(DEFAULT_ENGINE)) {
						selectedEngine = DEFAULT_ENGINE;
					} else {
						// If default engine is not available, then get the first engine of map returned
						selectedEngine = engines.keySet().iterator().next();
					}
					// Update application state with this engine
					Kernel.getInstance().state.setPlayerEngine(DEFAULT_ENGINE);
				}

				instance = engines.get(selectedEngine);

				// Init engine
				instance.initializePlayerEngine();
			}
		}
		return instance;
	}

	/**
	 * Return list of engine names as configured in settings file This method is
	 * mainly designed to be used in preferences window to select a player
	 * engine by its name
	 * 
	 * @return
	 */
	public final String[] getEngineNames() {
		return engineNames != null ? Arrays.copyOf(engineNames, engineNames.length) : null;
	}

	/**
	 * Starts playing current audio object from play list
	 * 
	 * @param buttonPressed
	 *            TODO: Add more javadoc
	 */
	public final void playCurrentAudioObject(boolean buttonPressed) {
		if (isEnginePlaying() && buttonPressed) { // Pause
			try {
				paused = true;
				VisualHandler.getInstance().setPlaying(false);
				pausePlayback();
				VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("PAUSED"));
				VisualHandler.getInstance().updateTitleBar("");
				VisualHandler.getInstance().updatePlayState(PlayState.PAUSED);
				logger.info(LogCategories.PLAYER, "Pause");
			} catch (Exception e) {
				notifyPlayerError(e);
				stopCurrentAudioObject(false);
			}
		} else {
			// If list is empty, is not playing
			VisualHandler.getInstance().setPlaying(MultiplePlaylistHandler.getInstance().getCurrentPlayList().size() > 0);

			AudioObject nextAudioObject = null;
			try {
				if (paused && buttonPressed) {
					paused = false;
					resumePlayback();
					if (!MultiplePlaylistHandler.getInstance().getCurrentPlayList().isEmpty()) {
						updateStatusBar(MultiplePlaylistHandler.getInstance().getCurrentPlayList().get(
								MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex()));
						VisualHandler.getInstance().updateTitleBar(
								MultiplePlaylistHandler.getInstance().getCurrentPlayList().get(
										MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex()));
						VisualHandler.getInstance().updatePlayState(PlayState.PAUSED);
						logger.info(LogCategories.PLAYER, "Resumed paused song");
					}
				} else {
					nextAudioObject = MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject();
					if (nextAudioObject != null) {
						if (isEnginePlaying() || paused) {
							stopCurrentAudioObject(false);
						}

						PlayListHandler.getInstance().selectedAudioObjectChanged(nextAudioObject);
						playAudioObject(nextAudioObject);
						// Setting volume and balance
						if (Kernel.getInstance().state.isMuteEnabled()) {
							applyMuteState(true);
						} else {
							setVolume(Kernel.getInstance().state.getVolume());
						}

						updateStatusBar(nextAudioObject);
						VisualHandler.getInstance().updateTitleBar(nextAudioObject);
						playListController.setSelectedSong(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex());
						VisualHandler.getInstance().setPlaying(true);
						logger.info(LogCategories.PLAYER, StringUtils.getString("Started play of file ", nextAudioObject));
					}
				}
				VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
			} catch (Exception e) {
				notifyPlayerError(e);
				stopCurrentAudioObject(false);
			}
		}

	}

	/**
	 * Stops playing current audio object
	 * 
	 * @param userStopped
	 *            <code>true</code> if user has stopped playback
	 * 
	 */
	public final void stopCurrentAudioObject(boolean userStopped) {
		try {
			boolean activateFadeAway = userStopped && Kernel.getInstance().state.isUseFadeAway() && !paused;

			VisualHandler.getInstance().setPlaying(false);
			if (paused) {
				paused = false;
			}

			submitToLastFmAndUpdateStats();

			stopPlayback(userStopped, activateFadeAway);
			setCurrentAudioObjectPlayedTime(0);

			VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("STOPPED"));
			VisualHandler.getInstance().updateTitleBar("");
			VisualHandler.getInstance().updatePlayState(PlayState.STOPPED);

			logger.info(LogCategories.PLAYER, "Stop");
		} catch (Exception e) {
			notifyPlayerError(e);
		}

	}

	/**
	 * Starts playing previous audio object from play list
	 */
	public final void playPreviousAudioObject() {
		switchPlaybackTo(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getPreviousAudioObjectToPlay(), false, false);
	}

	/**
	 * Starts playing next audio object from play list
	 * 
	 * @param autoNext
	 *            <code>true</code> if this method is called because current
	 *            audio object has finished, <code>false</code> if this method
	 *            is called because user has pressed the "NEXT" button
	 * 
	 */
	public final void playNextAudioObject(boolean autoNext) {
		switchPlaybackTo(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getNextAudioObjectToPlay(), true, autoNext);
	}

	/**
	 * Seek function: play current audio object from position defined by
	 * parameter (0-100%)
	 * 
	 * @param position
	 *            From start of audio object (0) to end of audio object (100)
	 * 
	 */
	public final void seekCurrentAudioObject(double position) {
		if (paused) {
			paused = false;
			if (!MultiplePlaylistHandler.getInstance().getCurrentPlayList().isEmpty()) {
				updateStatusBar(MultiplePlaylistHandler.getInstance().getCurrentPlayList().get(
						MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex()));
				VisualHandler.getInstance().updateTitleBar(
						MultiplePlaylistHandler.getInstance().getCurrentPlayList().get(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex()));
				VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
				VisualHandler.getInstance().setPlaying(true);
				logger.info(LogCategories.PLAYER, "Resumed paused song");
			}
		} else {
			seekTo(position);
		}
	}

	/**
	 * Sets position to play in current play list
	 * 
	 * @param pos
	 *            the new play list position to play
	 */
	public final void setPlayListPositionToPlay(int pos) {
		MultiplePlaylistHandler.getInstance().getCurrentPlayList().setCurrentAudioObject(pos);
	}

	/**
	 * Lower volume
	 */
	public final void volumeDown() {
		logger.debug(LogCategories.PLAYER);
		Kernel.getInstance().state.setVolume(Kernel.getInstance().state.getVolume() - 5);
		setVolume(Kernel.getInstance().state.getVolume());
		VisualHandler.getInstance().setVolume(Kernel.getInstance().state.getVolume());
	}

	/**
	 * Raise volume
	 */
	public final void volumeUp() {
		logger.debug(LogCategories.PLAYER);
		Kernel.getInstance().state.setVolume(Kernel.getInstance().state.getVolume() + 5);
		setVolume(Kernel.getInstance().state.getVolume());
		VisualHandler.getInstance().setVolume(Kernel.getInstance().state.getVolume());
	}

	/**
	 * Called when a exception is thrown related with player engine
	 * 
	 * @param e
	 *            The exception thrown
	 * 
	 */
	// TODO: This method should be protected
	public final static void notifyPlayerError(Exception e) {
		logger.error(LogCategories.PLAYER, StringUtils.getString("Player Error: ", e));
		logger.error(LogCategories.PLAYER, e);
		VisualHandler.getInstance().showErrorDialog(e.getMessage());
	}

	@Override
	public void applicationFinish() {
		instance.finishPlayer();
	}

	/**
	 * Instantiates a new player handler.
	 */
	protected PlayerHandler() {
		// Nothing to do
		// To properly init player must call method "initPlayerEngine"
	}

	/**
	 * Returns equalizer values or <code>null</code> if equalizer is disabled
	 * TODO: Add more explanation about equalizer values
	 * 
	 * @return the equalizer
	 */
	protected final float[] getEqualizerValues() {
		float[] equalizerSettings = Kernel.getInstance().state.getEqualizerSettings();
		return equalizerSettings != null ? Arrays.copyOf(equalizerSettings, equalizerSettings.length) : null;
	}

	/**
	 * Checks if sound normalization is enabled (<code>true</code>) or not (
	 * <code>false</code>)
	 * 
	 * @return <code>true</code> if sound normalization is enabled
	 */
	protected final boolean isSoundNormalizationEnabled() {
		return Kernel.getInstance().state.isUseNormalisation();
	}

	/**
	 * Checks if mute is enabled (<code>true</code>) or not (<code>false</code>)
	 * 
	 * @return <code>true</code> if mute is enabled
	 */
	protected final boolean isMuteEnabled() {
		return Kernel.getInstance().state.isMuteEnabled();
	}

	/**
	 * Sets the length of the current audio object
	 * 
	 * @param currentLength
	 *            The length of the current audio object in milliseconds (ms)
	 * 
	 */
	protected final void setCurrentAudioObjectLength(long currentLength) {
		this.currentAudioObjectLength = currentLength;
		// Update sliders with max length
		ControllerProxy.getInstance().getPlayerControlsController().setMaxDuration(currentLength);
		VisualHandler.getInstance().getFullScreenFrame().setAudioObjectLength(currentLength);
	}

	/**
	 * Sets the time played for the current audio object as playback advances
	 * 
	 * @param playedTime
	 *            played time in milliseconds (ms)
	 * 
	 */
	protected final void setCurrentAudioObjectPlayedTime(long playedTime) {
		this.playedTime = playedTime;
		ControllerProxy.getInstance().getPlayerControlsController().setTime(playedTime, currentAudioObjectLength);
		VisualHandler.getInstance().getFullScreenFrame().setCurrentAudioObjectPlayedTime(playedTime, currentAudioObjectLength);

		if (!submitToLastFmAndUpdateStats && (playedTime > currentAudioObjectLength / 2 || playedTime >= 240000)) {
			submitToLastFmAndUpdateStats = true;
		}
	}

	/**
	 * Submits the current audio object to Last.fm and updates stats
	 */
	private void submitToLastFmAndUpdateStats() {
		if (submitToLastFmAndUpdateStats && audioObject instanceof AudioFile) {
			ContextInformationServiceHandler.getInstance().submitToLastFm((AudioFile) audioObject, playedTime / 1000);
			RepositoryHandler.getInstance().setSongStatistics((AudioFile) audioObject);
			if (VisualHandler.getInstance().getStatsDialog().isVisible()) {
				ControllerProxy.getInstance().getStatsDialogController().updateStats();
			}
		}
	}

	/**
	 * Checks if playback is paused.
	 * 
	 * @return true, if is paused
	 */
	protected final boolean isPaused() {
		return paused;
	}

	/**
	 * Notifies the handler that the radio or podcast feed entry has started
	 * playing (MPlayer bug workaround).
	 */
	protected final void notifyRadioOrPodcastFeedEntryStarted() {
		logger.debug(LogCategories.PLAYER, "radio or podcast feed entry has started playing");
		// send volume command
		setVolume(Kernel.getInstance().state.getVolume());
		// if muted set mute again
		if (Kernel.getInstance().state.isMuteEnabled()) {
			applyMuteState(true);
		}
		logger.debug(LogCategories.PLAYER, "MPlayer bug (ignoring muting and volume settings after streamed file starts playing) workaround applied");
	}

	/**
	 * Actions to initialize engine. This method is called just after select an
	 * available player engine
	 */
	private final void initializePlayerEngine() {
		logger.debug(LogCategories.PLAYER);
		Kernel.getInstance().addFinishListener(this);
		this.playListController = ControllerProxy.getInstance().getPlayListController();
		VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("STOPPED"));
	}

	/**
	 * Returns a map of player engines read from settings file
	 * 
	 * @return Map with player engine name as key, and PlayerHandler as value
	 */
	private final static Map<String, PlayerHandler> getEnginesFromConfig() {
		Map<String, PlayerHandler> enginesMap = new HashMap<String, PlayerHandler>();
		try {
			// Load engine configuration
			PropertyResourceBundle enginesBundle = new PropertyResourceBundle(new FileInputStream(Constants.PLAYER_ENGINES_FILE));
			Enumeration<String> keys = enginesBundle.getKeys();

			while (keys.hasMoreElements()) {
				String engineName = keys.nextElement();
				String engineClassName = enginesBundle.getString(engineName);
				// Instance every engine and add to map
				enginesMap.put(engineName, (PlayerHandler) Class.forName(engineClassName).newInstance());
			}
		} catch (IOException e) {
			logger.error(LogCategories.PLAYER, e);
		} catch (ClassNotFoundException e) {
			logger.error(LogCategories.PLAYER, e);
		} catch (IllegalAccessException e) {
			logger.error(LogCategories.PLAYER, e);
		} catch (InstantiationException e) {
			logger.error(LogCategories.PLAYER, e);
		}
		return enginesMap;
	}

	/**
	 * Update status bar.
	 * 
	 * @param file
	 *            the file TODO: MOve this method to VisualHandler
	 */
	private final void updateStatusBar(AudioObject file) {
		if (!(file instanceof Radio || file instanceof PodcastFeedEntry)) {
			VisualHandler.getInstance().updateStatusBar(file);
		} else {
			VisualHandler.getInstance().updateStatusBar(StringUtils.getString("<html>", LanguageTool.getString("BUFFERING"), " <b>", file.getTitle(), "</b>...</html>"));
		}

	}

	/**
	 * TODO: Add documentation
	 * 
	 * @param audioObject
	 */
	private final void playAudioObject(AudioObject audioObject) {
		// If cacheFilesBeforePlaying is true and audio object is an audio file, copy it to temp folder
		// and start mplayer process from this copied file
		AudioObject audioObjectToPlay = null;
		if (audioObject instanceof AudioFile && Kernel.getInstance().state.isCacheFilesBeforePlaying()) {
			// Remove previous cached file
			if (lastFileCached != null) {
				TempFolder.getInstance().removeFile(lastFileCached);
			}

			File tempFile = TempFolder.getInstance().copyToTempFolder((File) audioObject);
			if (tempFile != null) {
				audioObjectToPlay = new AudioFile(tempFile.getAbsolutePath());
				lastFileCached = tempFile;
			} else {
				lastFileCached = null;
			}
		} else {
			audioObjectToPlay = audioObject;
		}

		// This audio object has not been listened yet
		submitToLastFmAndUpdateStats = false;

		// Show OSD
		if (Kernel.getInstance().state.isShowOSD()) {
			VisualHandler.getInstance().showOSDDialog();
		}

		// If we are playing a podcast, mark entry as listened
		if (audioObject instanceof PodcastFeedEntry) {
			((PodcastFeedEntry) audioObject).setListened(true);
			// Update pod cast navigator
			ControllerProxy.getInstance().getNavigationController().refreshPodcastFeedTreeContent();
		}

		// Send Now Playing info to Last.fm
		if (audioObject instanceof AudioFile) {
			ContextInformationServiceHandler.getInstance().submitNowPlayingInfoToLastFm((AudioFile) audioObject);
		}

		this.audioObject = audioObject;

		startPlayback(audioObjectToPlay, audioObject);
	}

	/**
	 * Starts playing audio object passed as parameter
	 * 
	 * @param audioObject
	 *            The audio object to play
	 * @param resetIfNoObject
	 *            If <code>true</code> reset play list position if audio object
	 *            is null
	 * @param autoNext
	 *            <code>true</code> if this method is called because of previous
	 *            audio object has finished playing or <code>false</code> if
	 *            it's called because of an action of the user (previous, next,
	 *            ...)
	 */
	private final void switchPlaybackTo(AudioObject audioObject, boolean resetIfNoObject, boolean autoNext) {
		boolean wasStopped = false;
		if (audioObject != null) {
			try {
				PlayListHandler.getInstance().selectedAudioObjectChanged(audioObject);
				if (isEnginePlaying() || isPaused() || autoNext) {
					stopCurrentAudioObject(false);
					if (!isPaused()) {
						playAudioObject(audioObject);
						VisualHandler.getInstance().setPlaying(true);
						if (Kernel.getInstance().state.isMuteEnabled()) {
							applyMuteState(true);
						} else {
							setVolume(Kernel.getInstance().state.getVolume());
						}
						logger.info(LogCategories.PLAYER, StringUtils.getString("Started play of file ", audioObject));
					} else {
						setCurrentAudioObjectPlayedTime(0);
						wasStopped = true;
					}
				} else {
					wasStopped = true;
					setCurrentAudioObjectPlayedTime(0);
				}
				playListController.setSelectedSong(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex());
				if (!wasStopped) {
					updateStatusBar(audioObject);
					VisualHandler.getInstance().updateTitleBar(audioObject);
					VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
					logger.debug(LogCategories.PLAYER, Integer.toString(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex()));
				}
			} catch (Exception e) {
				notifyPlayerError(e);
				stopCurrentAudioObject(false);
			}
		} else {
			if (resetIfNoObject) {
				stopCurrentAudioObject(false);
				MultiplePlaylistHandler.getInstance().getCurrentPlayList().setCurrentAudioObject(0);
				playListController.setSelectedSong(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObjectIndex());
				PlayListHandler.getInstance().selectedAudioObjectChanged(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject());
			}
		}
	}

	///////////////////////////////////////////////// GENERAL ATTRIBUTES //////////////////////////////////////////////////

	/**
	 * The player engine used
	 */
	private static PlayerHandler instance;

	/**
	 * The logger used in player engines
	 */
	protected static Logger logger = new Logger();

	/**
	 * The play list controller
	 */
	private PlayListController playListController;

	/**
	 * Default player engine
	 */
	public static final String DEFAULT_ENGINE = "MPlayer";

	/**
	 * Names of all engines
	 */
	private static String[] engineNames;

	///////////////////////////////////////////////// PLAYLIST ATTRIBUTES /////////////////////////////////////////////////

	/**
	 * Paused property: <code>true</code> if audio object playback is paused
	 */
	private boolean paused;

	///////////////////////////////////////////////// CURRENT AUDIO OBJECT ATTRIBUTES /////////////////////////////////////

	/**
	 * Audio object
	 */
	private AudioObject audioObject;

	/**
	 * Length of the current audio object
	 */
	private long currentAudioObjectLength;

	/**
	 * This attribute is used when caching files option is enabled. When caching
	 * a new file, file pointed by this object is removed from temp folder
	 */
	private File lastFileCached;

	/**
	 * A flag indicating the current file has been listened (played half length
	 * or more than four minutes) NOTE: It must be set to false when starting
	 * playing every audio object
	 */
	private boolean submitToLastFmAndUpdateStats = false;

	/**
	 * The time the current audio object has already played
	 */
	private long playedTime;

}
