/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.contextInformation.lastfm;

import java.awt.Image;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.ImageIcon;

import net.roarsoftware.lastfm.Album;
import net.roarsoftware.lastfm.Artist;
import net.roarsoftware.lastfm.Caller;
import net.roarsoftware.lastfm.Playlist;
import net.roarsoftware.lastfm.scrobble.ResponseStatus;
import net.roarsoftware.lastfm.scrobble.Scrobbler;
import net.roarsoftware.lastfm.scrobble.Source;
import net.roarsoftware.lastfm.scrobble.SubmissionData;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.modules.contextInformation.AlbumInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.AlbumListInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.ArtistInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.SimilarArtistsInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.lastfm.data.LastFmAlbum;
import net.sourceforge.atunes.kernel.modules.contextInformation.lastfm.data.LastFmAlbumList;
import net.sourceforge.atunes.kernel.modules.contextInformation.lastfm.data.LastFmSimilarArtists;
import net.sourceforge.atunes.kernel.modules.proxy.Proxy;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.AudioFilePictureUtils;
import net.sourceforge.atunes.kernel.utils.NetworkUtils;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class LastFmService.
 * 
 * This class is responsible of retrieve information from Last.fm web services.
 */
public class LastFmService {

	private static Logger logger = new Logger();

	private static final String API_KEY = "e4dd8c105b26ecbf84e2806d7a9660f3";
	private static final String CLIENT_ID = "atu";
	private static final String CLIENT_VERSION = "1.0";

	private static final String ARTIST_WILDCARD = "(%ARTIST%)";
	private static final String LANGUAGE_PARAM = "?setlang=";
	private static final String LANGUAGE_WILDCARD = "(%LANGUAGE%)";
	private static final String ARTIST_WIKI_URL = StringUtils.getString("http://www.lastfm.com/music/", ARTIST_WILDCARD, "/+wiki", LANGUAGE_PARAM, LANGUAGE_WILDCARD);

	private Proxy proxy;

	private Scrobbler scrobbler;
	private String user;
	private String password;
	private boolean handshakePerformed;

	/**
	 * Instantiates a new Last.fm service
	 * 
	 * @param proxy
	 *            the proxy
	 * @param user
	 *            the Last.fm username
	 * @param password
	 *            the Last.fm password
	 */
	public LastFmService(Proxy proxy, String user, String password) {
		this.proxy = proxy;
		this.user = user;
		this.password = password;
		Caller.getInstance().setProxy(proxy);
		Caller.getInstance().setUserAgent(CLIENT_ID);
		scrobbler = Scrobbler.newScrobbler(CLIENT_ID, CLIENT_VERSION, user);
		this.handshakePerformed = false;
	}

	/**
	 * Gets the album.
	 * 
	 * @param artist
	 *            the artist
	 * @param album
	 *            the album
	 * 
	 * @return the album
	 */
	public AlbumInfo getAlbum(String artist, String album) {
		try {
			// Try to get from cache
			AlbumInfo albumObject = LastFmCache.retrieveAlbumInfo(artist, album);
			if (albumObject == null) {
				Album a = Album.getInfo(artist, album, API_KEY);
				if (a != null) {
					Playlist pl = Playlist.fetchAlbumPlaylist(a.getId(), API_KEY);
					albumObject = LastFmAlbum.getAlbum(a, pl);
					LastFmCache.storeAlbumInfo(artist, album, albumObject);
				}
			}
			return albumObject;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the album list.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the album list
	 */
	public AlbumListInfo getAlbumList(String artist) {
		try {
			// Try to get from cache
			AlbumListInfo albumList = LastFmCache.retrieveAbumList(artist);
			if (albumList == null) {

				Collection<Album> as = Artist.getTopAlbums(artist, API_KEY);
				if (as != null) {
					AlbumListInfo albums = LastFmAlbumList.getAlbumList(as, artist);

					List<AlbumInfo> result = new ArrayList<AlbumInfo>();
					for (AlbumInfo a : albums.getAlbums()) {
						if (a.getBigCoverURL() != null && !a.getBigCoverURL().isEmpty()) {
							result.add(a);
						}
					}

					albumList = new LastFmAlbumList();
					albumList.setArtist(artist);
					albumList.setAlbums(result);
					LastFmCache.storeAlbumList(artist, albumList);
				}
			}
			return albumList;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the artist top tag.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the artist top tag
	 */
	public String getArtistTopTag(String artist) {
		try {
			Collection<String> topTags = Artist.getTopTags(artist, API_KEY);
			List<String> tags = new ArrayList<String>(topTags);
			return tags.isEmpty() ? "" : tags.get(0);
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the image.
	 * 
	 * @param album
	 *            the album
	 * 
	 * @return the image
	 */
	public Image getImage(AlbumInfo album, AudioFile file) {
		try {
			Image img = null;
			if (album.getTitle().trim().equalsIgnoreCase(file.getAlbum().trim()) && album.getArtist().trim().equalsIgnoreCase(file.getArtist().trim())) {
				ImageIcon imageIcon = AudioFilePictureUtils.getImageForAudioFile(file, -1, -1);
				img = imageIcon != null ? imageIcon.getImage() : null;
			}
			// Try to retrieve from cache
			if (img == null) {
				img = LastFmCache.retrieveAlbumCover(album);
			}
			if (img == null && album.getBigCoverURL() != null && !album.getBigCoverURL().isEmpty()) {
				img = NetworkUtils.getImage(NetworkUtils.getConnection(album.getBigCoverURL(), proxy));
				LastFmCache.storeAlbumCover(album, img);
			}

			return img;
		} catch (IOException e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the image.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the image
	 */
	public Image getImage(ArtistInfo artist) {
		try {
			// Try to retrieve from cache
			Image img = LastFmCache.retrieveArtistThumbImage(artist);
			if (img == null && artist.getImageUrl() != null && !artist.getImageUrl().isEmpty()) {
				img = NetworkUtils.getImage(NetworkUtils.getConnection(artist.getImageUrl(), proxy));
				LastFmCache.storeArtistThumbImage(artist, img);
			}
			return img;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the image.
	 * 
	 * @param similar
	 *            the similar
	 * 
	 * @return the image
	 */
	public Image getImage(SimilarArtistsInfo similar) {
		try {
			// Try to retrieve from cache
			Image img = LastFmCache.retrieveArtistImage(similar);
			if (img == null) {
				img = NetworkUtils.getImage(NetworkUtils.getConnection(similar.getPicture(), proxy));
				LastFmCache.storeArtistImage(similar, img);
			}
			return img;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the similar artists.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the similar artists
	 */
	public SimilarArtistsInfo getSimilarArtists(String artist) {
		try {
			// Try to get from cache
			SimilarArtistsInfo similar = LastFmCache.retrieveArtistSimilar(artist);
			if (similar == null) {
				Collection<Artist> as = Artist.getSimilar(artist, API_KEY);
				Artist a = Artist.getInfo(artist, API_KEY);
				if (a != null) {
					similar = LastFmSimilarArtists.getSimilarArtists(as, a);
					LastFmCache.storeArtistSimilar(artist, similar);
				}
			}
			return similar;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the wiki text.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the wiki text
	 */
	public String getWikiText(String artist) {
		try {
			// Try to get from cache
			String wikiText = LastFmCache.retrieveArtistWiki(artist);
			if (wikiText == null) {

				Artist a = Artist.getInfo(artist, API_KEY);
				wikiText = a != null ? a.getWikiSummary() : "";
				wikiText = wikiText.replaceAll("<.*?>", "");
				wikiText = StringUtils.unescapeHTML(wikiText, 0);

				LastFmCache.storeArtistWiki(artist, wikiText);
			}
			return wikiText;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
		}
		return null;
	}

	/**
	 * Gets the wiki url.
	 * 
	 * @param artist
	 *            the artist
	 * 
	 * @return the wiki url
	 */
	public String getWikiURL(String artist) {
		return ARTIST_WIKI_URL.replace(ARTIST_WILDCARD, NetworkUtils.encodeString(artist)).replace(LANGUAGE_WILDCARD, Kernel.getInstance().state.getLocale().getLanguage());
	}

	/**
	 * Submits song to Last.fm
	 * 
	 * @param file
	 *            audio file
	 * @param secondsPlayed
	 *            seconds the audio file has already played
	 * @throws ScrobblerException
	 */
	public void submit(AudioFile file, long secondsPlayed) throws ScrobblerException {
		if (!Kernel.getInstance().state.isLastFmEnabled()) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Last.fm disabled");
			return;
		}

		if (user == null || user.equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Empty user");
			return;
		}

		if (password == null || password.equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Empty password");
			return;
		}

		// Get started to play
		long startedToPlay = System.currentTimeMillis() / 1000 - secondsPlayed;

		// If artist unknown don't submit
		if (file.getArtist().equals(LanguageTool.getString("UNKNOWN_ARTIST"))) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Unknown Artist");
			return;
		}

		// If title unknown don't submit
		if (file.getTitle().equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Unknown Title");
			return;
		}

		// Do not submit tracks under 30 seconds
		if (file.getDuration() < 30) {
			logger.debug(LogCategories.SERVICE, "Don't submit to Last.fm: Lenght < 30");
			return;
		}

		logger.info(LogCategories.SERVICE, "Trying to submit song to Last.fm");
		try {
			performHandshakeIfNeeded();
			SubmissionData submissionData = new SubmissionData(file.getArtist(), file.getTitle(), file.getAlbum(), (int) file.getDuration(), file.getTrackNumber(), Source.USER,
					startedToPlay);
			ResponseStatus status = scrobbler.submit(submissionData);
			if (status.ok()) {
				logger.info(LogCategories.SERVICE, "Song submitted to Last.fm");
			} else {
				handshakePerformed = false;
				throw new ScrobblerException(status.getStatus());
			}

		} catch (IOException e) {
			logger.error(LogCategories.SERVICE, e);
			handshakePerformed = false;
			throw new ScrobblerException(e.getMessage());
		}
	}

	/**
	 * Submits now playing info to Last.fm
	 * 
	 * @param file
	 *            audio file
	 * @throws ScrobblerException
	 */
	public void submitNowPlayingInfo(AudioFile file) throws ScrobblerException {
		if (!Kernel.getInstance().state.isLastFmEnabled()) {
			logger.debug(LogCategories.SERVICE, "Don't submit now playing info to Last.fm: Last.fm disabled");
			return;
		}

		if (user == null || user.equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit now playing info to Last.fm: Empty user");
			return;
		}

		if (password == null || password.equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit now playing info to Last.fm: Empty password");
			return;
		}

		// If artist unknown don't submit
		if (file.getArtist().equals(LanguageTool.getString("UNKNOWN_ARTIST"))) {
			logger.debug(LogCategories.SERVICE, "Don't submit now playing info to Last.fm: Unknown Artist");
			return;
		}

		// If title unknown don't submit
		if (file.getTitle().equals("")) {
			logger.debug(LogCategories.SERVICE, "Don't submit now playing info to Last.fm: Unknown Title");
			return;
		}

		logger.info(LogCategories.SERVICE, "Trying to submit now playing info to Last.fm");
		try {
			performHandshakeIfNeeded();
			ResponseStatus status = scrobbler.nowPlaying(file.getArtist(), file.getTitle(), file.getAlbum(), (int) file.getDuration(), file.getTrackNumber());
			if (status.ok()) {
				logger.info(LogCategories.SERVICE, "Now playing info submitted to Last.fm");
			} else {
				handshakePerformed = false;
				throw new ScrobblerException(status.getStatus());
			}
		} catch (IOException e) {
			logger.error(LogCategories.SERVICE, e);
			handshakePerformed = false;
			throw new ScrobblerException(e.getMessage());
		}
	}

	/**
	 * Performs handshake for submissions if needed
	 * 
	 * @throws IOException
	 * @throws ScrobblerException
	 */
	private void performHandshakeIfNeeded() throws IOException, ScrobblerException {
		if (!handshakePerformed) {
			ResponseStatus status = scrobbler.handshake(password);
			if (!status.ok()) {
				throw new ScrobblerException(status.getStatus());
			}
			handshakePerformed = true;
		}
	}

}
