/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.repository.audio;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.sourceforge.atunes.kernel.modules.repository.tags.reader.TagDetector;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.DefaultTag;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.Tag;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

import org.jaudiotagger.audio.AudioFileIO;

/**
 * AudioFile class initializes audio files so tags and audio information can be
 * retrieved by tagging library. Provides information about bitrate, duration
 * and frequency of the audio file. Provides tag informations.
 * 
 * @author fleax
 */
public class AudioFile extends File implements AudioObject, Serializable {

	/** The logger. */
	private static transient Logger logger = new Logger();

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 7907490762633634241L;

	/** The Constant MP3_FORMAT. */
	public static final String MP3_FORMAT = "mp3";

	/** The Constant OGG_FORMAT. */
	public static final String OGG_FORMAT = "ogg";

	/** The Constant MP4_FORMAT_1. */
	public static final String MP4_FORMAT_1 = "m4a";

	/** The Constant MP4_FORMAT_2. */
	public static final String MP4_FORMAT_2 = "mp4";

	/** The Constant WAV_FORMAT. */
	public static final String WAV_FORMAT = "wav";

	/** The Constant WMA_FORMAT. */
	public static final String WMA_FORMAT = "wma";

	/** The Constant FLAC_FORMAT. */
	public static final String FLAC_FORMAT = "flac";

	/** The Constant APE_FORMAT. */
	public static final String APE_FORMAT = "ape";

	/** The Constant MPC_FORMAT. */
	public static final String MPC_FORMAT = "mpc";
	//public static final String AU_FORMAT = "au";
	/** The Constant REALAUDIO_FORMAT. */
	public static final String REALAUDIO_FORMAT = "ra";

	/** The Constant REALAUDIO_FORMAT_2. */
	public static final String REALAUDIO_FORMAT_2 = "rm";

	/** The Constant MPplus_FORMAT. */
	public static final String MPplus_FORMAT = "mp+";

	/** The Constant MAC_FORMAT. */
	public static final String MAC_FORMAT = "mac";
	//public static final String MP2_FORMAT = "mp2";

	/** The tag. */
	private Tag tag;

	/** The external pictures. */
	private List<File> externalPictures;

	/** The duration. */
	private long duration;

	/** The bitrate. */
	private long bitrate;

	/** The frequency. */
	private int frequency;

	/** The read time. */
	private long readTime;

	/** The stars. */
	private int stars = 0;

	/**
	 * Instantiates a new audio file.
	 * 
	 * @param fileName
	 *            the file name
	 */
	public AudioFile(String fileName) {
		super(fileName);
		introspectTags();
		readAudioProperties(this);
		readTime = System.currentTimeMillis();
	}

	/**
	 * Gets the audio files.
	 * 
	 * @param audioObjects
	 *            the audio objects
	 * 
	 * @return the audio files
	 */
	public static List<AudioFile> getAudioFiles(List<AudioObject> audioObjects) {
		List<AudioFile> result = new ArrayList<AudioFile>();
		for (AudioObject audioObject : audioObjects) {
			if (audioObject instanceof AudioFile) {
				result.add((AudioFile) audioObject);
			}
		}
		return result;
	}

	/**
	 * Gets the new tag.
	 * 
	 * @param file
	 *            the file
	 * @param properties
	 *            the properties
	 * 
	 * @return the new tag
	 */
	public static Tag getNewTag(AudioFile file, Map<String, Object> properties) {
		return new DefaultTag().getTagFromProperties(properties);
	}

	/**
	 * Checks if is ape file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is ape file
	 */
	public static boolean isApeFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(APE_FORMAT) || file.getAbsolutePath().toLowerCase().endsWith(MAC_FORMAT);
	}

	/**
	 * Checks if is flac file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is flac file
	 */
	public static boolean isFlacFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(FLAC_FORMAT);
	}

	/**
	 * Checks if is mp3 file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is mp3 file
	 */
	public static boolean isMp3File(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(MP3_FORMAT);
	}

	/**
	 * Checks if is mp4 file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is mp4 file
	 */
	public static boolean isMp4File(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(MP4_FORMAT_1) || file.getAbsolutePath().toLowerCase().endsWith(MP4_FORMAT_2);
	}

	/**
	 * Checks if is mPC file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is mPC file
	 */
	public static boolean isMPCFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(MPC_FORMAT) || file.getAbsolutePath().toLowerCase().endsWith(MPplus_FORMAT);
	}

	/**
	 * Checks if is ogg file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is ogg file
	 */
	public static boolean isOggFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(OGG_FORMAT);
	}

	/**
	 * Checks if is real audio file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is real audio file
	 */
	public static boolean isRealAudioFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(REALAUDIO_FORMAT) || file.getAbsolutePath().toLowerCase().endsWith(REALAUDIO_FORMAT_2);
	}

	/**
	 * Checks if is valid audio file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is valid audio file
	 */
	public static boolean isValidAudioFile(File file) {
		return !file.isDirectory()
				&& (isMp3File(file) || isOggFile(file) || isMp4File(file) || isWavFile(file) || isWmaFile(file) || isFlacFile(file) || isRealAudioFile(file) || isApeFile(file) || isMPCFile(file));
	}

	/**
	 * Checks if is valid audio file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is valid audio file
	 */
	public static boolean isValidAudioFile(String file) {
		File f = new File(file);
		return f.exists() && isValidAudioFile(f);
	}

	/**
	 * Checks if is wav file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is wav file
	 */
	public static boolean isWavFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(WAV_FORMAT);
	}

	/**
	 * Checks if is wma file.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return true, if is wma file
	 */
	public static boolean isWmaFile(File file) {
		return file.getAbsolutePath().toLowerCase().endsWith(WMA_FORMAT);
	}

	/**
	 * Read audio properties.
	 * 
	 * @param file
	 *            the file
	 */
	private static void readAudioProperties(AudioFile file) {
		try {
			org.jaudiotagger.audio.AudioFile f = AudioFileIO.read(file);
			file.duration = f.getAudioHeader().getTrackLength();
			file.bitrate = f.getAudioHeader().getBitRateAsNumber();
			file.frequency = f.getAudioHeader().getSampleRateAsNumber();
		} catch (Exception e) {
			logger.error(LogCategories.FILE_READ, e.getMessage());
		}
	}

	/**
	 * Adds the external picture.
	 * 
	 * @param picture
	 *            the picture
	 */
	public void addExternalPicture(File picture) {
		if (externalPictures != null && !externalPictures.contains(picture)) {
			externalPictures.add(0, picture);
		}
	}

	/**
	 * Delete tags.
	 */
	private void deleteTags() {
		tag = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.File#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object o) {
		if (o == null || !(o instanceof AudioFile)) {
			return false;
		}
		return ((AudioFile) o).getUrl().equals(getUrl());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.File#getAbsolutePath()
	 */
	@Override
	@Deprecated
	public String getAbsolutePath() {
		return super.getAbsolutePath();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getAlbum()
	 */
	@Override
	public String getAlbum() {
		String album;
		if (tag != null && tag.getAlbum() != null && !tag.getAlbum().isEmpty()) {
			album = tag.getAlbum();
		} else {
			album = LanguageTool.getString("UNKNOWN_ALBUM");
		}
		return album;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getAlbumArtist()
	 */
	@Override
	public String getAlbumArtist() {
		String albumArtist;
		if (tag != null && tag.getAlbumArtist() != null) {
			albumArtist = tag.getAlbumArtist();
		} else {
			albumArtist = "";
		}
		return albumArtist;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getArtist()
	 */
	@Override
	public String getArtist() {
		String artist;
		if (tag != null && tag.getArtist() != null && !tag.getArtist().isEmpty()) {
			artist = tag.getArtist();
		} else {
			artist = LanguageTool.getString("UNKNOWN_ARTIST");
		}
		return artist;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getBitrate()
	 */
	@Override
	public long getBitrate() {
		return bitrate;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getComposer()
	 */
	@Override
	public String getComposer() {
		String composer;
		if (tag != null && tag.getComposer() != null) {
			composer = tag.getComposer();
		} else {
			composer = "";
		}
		return composer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getDuration()
	 */
	@Override
	public long getDuration() {
		return duration;
	}

	/**
	 * Gets the external pictures.
	 * 
	 * @return the external pictures
	 */
	public List<File> getExternalPictures() {
		return externalPictures;
	}

	/**
	 * Gets the external pictures count.
	 * 
	 * @return the external pictures count
	 */
	public int getExternalPicturesCount() {
		return externalPictures != null ? externalPictures.size() : 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getFrequency()
	 */
	@Override
	public int getFrequency() {
		return frequency;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getGenre()
	 */
	@Override
	public String getGenre() {
		if (tag != null && tag.getGenre() != null) {
			return tag.getGenre();
		}
		return LanguageTool.getString("UNKNOWN_GENRE");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getLyrics()
	 */
	@Override
	public String getLyrics() {
		String lyrics;
		if (tag != null && tag.getLyrics() != null) {
			lyrics = tag.getLyrics();
		} else {
			lyrics = "";
		}
		return lyrics;
	}

	/**
	 * Return tag comment
	 * 
	 * @return
	 */
	public String getComment() {
		if (tag != null && tag.getComment() != null) {
			return tag.getComment();
		}
		return "";
	}

	/**
	 * Gets the name without extension.
	 * 
	 * @return the name without extension
	 */
	public String getNameWithoutExtension() {
		if (getName().indexOf('.') != -1) {
			return getName().substring(0, getName().lastIndexOf('.'));
		}
		return getName();
	}

	/**
	 * Gets the stars.
	 * 
	 * @return the stars
	 */
	@Override
	public int getStars() {
		return stars;
	}

	/**
	 * Gets the tag.
	 * 
	 * @return the tag
	 */
	public Tag getTag() {
		return tag;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getTitle()
	 */
	@Override
	public String getTitle() {
		String title;
		if (tag != null && tag.getTitle() != null) {
			title = tag.getTitle();
		} else {
			title = "";
		}
		return title;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getTitleOrFileName()
	 */
	@Override
	public String getTitleOrFileName() {
		String title;
		if (tag != null && tag.getTitle() != null && !tag.getTitle().isEmpty()) {
			title = tag.getTitle();
		} else {
			title = getNameWithoutExtension();
		}
		return title;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getTrackNumber()
	 */

	@Override
	public Integer getTrackNumber() {
		if (tag != null) {
			if (tag instanceof DefaultTag) {
				return ((DefaultTag) tag).getTrackNumber();
			}
			return 0;
		}
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getUrl()
	 */
	@Override
	public String getUrl() {
		return super.getAbsolutePath();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#getYear()
	 */
	@Override
	public String getYear() {
		if (tag != null && tag.getYear() > 0) {
			return Integer.toString(tag.getYear());
		}
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.File#hashCode()
	 */
	@Override
	public int hashCode() {
		return getUrl().hashCode();
	}

	/**
	 * Checks for internal picture.
	 * 
	 * @return true, if successful
	 */
	public final boolean hasInternalPicture() {
		return tag != null && tag.hasInternalImage();
	}

	/**
	 * Checks if the tag of this audio file does support internal images
	 * 
	 * @return if the tag of this audio file does support internal images
	 */
	public final boolean supportsInternalPicture() {
		return tag != null && tag.isSupportsInternalImage();
	}

	/**
	 * Introspect tags.
	 */
	private void introspectTags() {
		TagDetector.getTags(this);
	}

	/**
	 * Checks if is up to date.
	 * 
	 * @return true, if is up to date
	 */
	public boolean isUpToDate() {
		return readTime > lastModified();
	}

	/**
	 * Refresh tag.
	 */
	public void refreshTag() {
		deleteTags();
		introspectTags();
		readTime = System.currentTimeMillis();
	}

	/**
	 * Sets the external pictures.
	 * 
	 * @param externalPictures
	 *            the new external pictures
	 */
	public void setExternalPictures(List<File> externalPictures) {
		this.externalPictures = externalPictures;
	}

	/**
	 * Sets the stars.
	 * 
	 * @param stars
	 *            the stars to set
	 */
	@Override
	public void setStars(int stars) {
		this.stars = stars;
	}

	/**
	 * Sets the tag.
	 * 
	 * @param tag
	 *            the new tag
	 */
	public void setTag(Tag tag) {
		this.tag = tag;
	}

	/**
	 * Sets write permissions if is not writable.
	 */
	public void setWritable() {
		// Set write permission on file
		if (!canWrite()) {
			setWritable(true);
		}
		// Set write permission on parent
		if (!getParentFile().canWrite()) {
			getParentFile().setWritable(true);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.File#toString()
	 */
	@Override
	public String toString() {
		return getName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.model.AudioObject#isSeekable()
	 */
	@Override
	public boolean isSeekable() {
		return true;
	}
}
