/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.repository.tags.tag.image;

import java.io.ByteArrayInputStream;
import java.util.List;

import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.flac.metadatablock.MetadataBlockDataPicture;
import org.jaudiotagger.tag.KeyNotFoundException;
import org.jaudiotagger.tag.Tag;
import org.jaudiotagger.tag.TagField;
import org.jaudiotagger.tag.TagFieldKey;
import org.jaudiotagger.tag.datatype.DataTypes;
import org.jaudiotagger.tag.flac.FlacTag;
import org.jaudiotagger.tag.id3.AbstractID3v2Frame;
import org.jaudiotagger.tag.id3.AbstractID3v2Tag;
import org.jaudiotagger.tag.id3.framebody.FrameBodyAPIC;
import org.jaudiotagger.tag.mp4.Mp4FieldKey;
import org.jaudiotagger.tag.mp4.Mp4Tag;
import org.jaudiotagger.tag.mp4.Mp4TagField;
import org.jaudiotagger.tag.mp4.field.Mp4TagCoverField;
import org.jaudiotagger.tag.reference.PictureTypes;
import org.jaudiotagger.tag.vorbiscomment.VorbisCommentFieldKey;
import org.jaudiotagger.tag.vorbiscomment.VorbisCommentTag;
import org.jaudiotagger.tag.vorbiscomment.util.Base64Coder;

/**
 * Tagger for internal pictures
 */
public class InternalPictureTagger {

	private static Logger logger = new Logger();

	/**
	 * Adds a picture to a tag
	 * 
	 * @param image
	 *            the picture that should be added, if null all pictures will be
	 *            deleted
	 * @param tag
	 *            the tag
	 */
	public static void addPictureToTag(byte[] image, Tag tag) {
		try {
			// Remove old images
			if (tag instanceof VorbisCommentTag) {
				List<TagField> list = ((VorbisCommentTag) tag).get(VorbisCommentFieldKey.COVERART);
				int i = list != null ? list.size() : 0;
				while (i > 0) {
					((VorbisCommentTag) tag).deleteTagField(VorbisCommentFieldKey.COVERART);
					i--;
				}
				List<TagField> list2 = ((VorbisCommentTag) tag).get(VorbisCommentFieldKey.COVERARTMIME);
				i = list2 != null ? list2.size() : 0;
				while (i > 0) {
					((VorbisCommentTag) tag).deleteTagField(VorbisCommentFieldKey.COVERARTMIME);
					i--;
				}
			} else if (tag instanceof Mp4Tag) {
				List<TagField> list = ((Mp4Tag) tag).get(Mp4FieldKey.ARTWORK);
				int i = list != null ? list.size() : 0;
				while (i > 0) {
					((Mp4Tag) tag).deleteTagField(Mp4FieldKey.ARTWORK);
					i--;
				}
			} else {
				List<TagField> list = tag.get(TagFieldKey.COVER_ART);
				int i = list != null ? list.size() : 0;
				while (i > 0) {
					tag.deleteTagField(TagFieldKey.COVER_ART);
					i--;
				}
			}
		} catch (KeyNotFoundException e) {
			logger.error(LogCategories.IMAGE, e);
		}

		// Write new image
		if (image != null) {
			ImageInfo imageInfo = new ImageInfo();
			imageInfo.setInput(new ByteArrayInputStream(image));
			String mimeType = imageInfo.getMimeType();
			int width = imageInfo.getWidth();
			int height = imageInfo.getHeight();
			int colorDepth = imageInfo.getBitsPerPixel();

			try {
				if (tag instanceof AbstractID3v2Tag) {
					tag.set(((AbstractID3v2Tag) tag).createArtworkField(image, mimeType));
				} else if (tag instanceof Mp4Tag) {
					tag.set(((Mp4Tag) tag).createArtworkField(image));
				} else if (tag instanceof VorbisCommentTag) {
					char[] testdata = Base64Coder.encode(image);
					String base64image = new String(testdata);
					tag.set(((VorbisCommentTag) tag).createTagField(VorbisCommentFieldKey.COVERART, base64image));
					tag.set(((VorbisCommentTag) tag).createTagField(VorbisCommentFieldKey.COVERARTMIME, mimeType));
				} else if (tag instanceof FlacTag) {
					tag.set(((FlacTag) tag).createArtworkField(image, PictureTypes.DEFAULT_ID, mimeType, "cover", width, height, colorDepth, 0));
				}
			} catch (Exception e) {
				logger.error(LogCategories.IMAGE, e);
			}
		}
	}

	/**
	 * Checks if a tag has an internal picture
	 * 
	 * @param tag
	 *            Tag
	 * @return if a tag has an internal picture
	 */
	public static boolean hasInternalPicture(Tag tag) {
		if (tag instanceof AbstractID3v2Tag) {
			List<TagField> imageFields = tag.get(org.jaudiotagger.tag.TagFieldKey.COVER_ART);
			return imageFields != null && imageFields.size() >= 1;
		} else if (tag instanceof Mp4Tag) {
			TagField coverArtField = ((Mp4Tag) tag).getFirstField(Mp4FieldKey.ARTWORK);
			return coverArtField != null;
		} else if (tag instanceof VorbisCommentTag) {
			byte[] b = ((VorbisCommentTag) tag).getArtworkBinaryData();
			return b != null && b.length > 0;
		} else if (tag instanceof FlacTag) {
			List<MetadataBlockDataPicture> images = ((FlacTag) tag).getImages();
			return images != null && !images.isEmpty();
		} else {
			return false;
		}

	}

	/**
	 * Returns image stored into audio file as byte array, if exists.
	 * 
	 * @param file
	 *            the file
	 * 
	 * @return the inside picture as byte array
	 */
	public static byte[] getInsidePictureAsByteArray(AudioFile file) {
		try {
			if (file.hasInternalPicture()) {
				Tag tag = AudioFileIO.read(file).getTag();

				if (tag instanceof AbstractID3v2Tag) {
					logger.debug(LogCategories.IMAGE, "ID3 image found");
					List<TagField> imageFields = tag.get(TagFieldKey.COVER_ART);
					TagField imageField = imageFields != null && !imageFields.isEmpty() ? imageFields.get(0) : null;
					if (imageField != null) {
						FrameBodyAPIC imageFrameBody = (FrameBodyAPIC) ((AbstractID3v2Frame) imageField).getBody();
						if (!imageFrameBody.isImageUrl()) {
							return (byte[]) imageFrameBody.getObjectValue(DataTypes.OBJ_PICTURE_DATA);
						}
					}
				} else if (tag instanceof Mp4Tag) {
					logger.debug(LogCategories.IMAGE, "Mp4 image found");
					Mp4TagField coverArtField = ((Mp4Tag) tag).getFirstField(Mp4FieldKey.ARTWORK);
					return coverArtField == null ? null : ((Mp4TagCoverField) coverArtField).getData();
				} else if (tag instanceof VorbisCommentTag) {
					logger.debug(LogCategories.IMAGE, "Vorbiscomment image found");
					return ((VorbisCommentTag) tag).getArtworkBinaryData();
				} else if (tag instanceof FlacTag) {
					logger.debug(LogCategories.IMAGE, "Flac image found");
					List<MetadataBlockDataPicture> images = ((FlacTag) tag).getImages();
					if (images != null && !images.isEmpty()) {
						return images.get(0).getImageData();
					} else {
						return null;
					}
				}

			}
			return null;
		} catch (Exception e) {
			logger.error(LogCategories.IMAGE, e);
			return null;
		}
	}

	/**
	 * Checks if a tag does support internal images
	 * 
	 * @param tag
	 *            tag
	 * @return if a tag does support internal images
	 */
	public static boolean supportsInternalPicture(Tag tag) {
		return tag instanceof AbstractID3v2Tag || tag instanceof Mp4Tag || tag instanceof VorbisCommentTag || tag instanceof FlacTag;
	}

}
